from migen.fhdl.module import Module
from migen.fhdl.specials import Instance, Tristate
from migen.fhdl.bitcontainer import value_bits_sign
from migen.genlib.io import *
from migen.genlib.resetsync import AsyncResetSynchronizer


class LatticeECPXAsyncResetSynchronizerImpl(Module):
    def __init__(self, cd, async_reset):
        rst1 = Signal()
        self.specials += [
            Instance("FD1S3BX", i_D=0, i_PD=async_reset,
                     i_CK=cd.clk, o_Q=rst1),
            Instance("FD1S3BX", i_D=rst1, i_PD=async_reset,
                     i_CK=cd.clk, o_Q=cd.rst)
        ]


class LatticeECPXAsyncResetSynchronizer:
    @staticmethod
    def lower(dr):
        return LatticeECPXAsyncResetSynchronizerImpl(dr.cd, dr.async_reset)


class LatticeECPXDDROutputImpl(Module):
    def __init__(self, i1, i2, o, clk):
        self.specials += Instance("ODDRXD1",
                                  synthesis_directive="ODDRAPPS=\"SCLK_ALIGNED\"",
                                  i_SCLK=clk,
                                  i_DA=i1, i_DB=i2, o_Q=o)


class LatticeECPXDDROutput:
    @staticmethod
    def lower(dr):
        return LatticeECPXDDROutputImpl(dr.i1, dr.i2, dr.o, dr.clk)

lattice_ecpx_special_overrides = {
    AsyncResetSynchronizer: LatticeECPXAsyncResetSynchronizer,
    DDROutput: LatticeECPXDDROutput
}


class LatticeECPXTrellisTristateImpl(Module):
    def __init__(self, io, o, oe, i):
        nbits, sign = value_bits_sign(io)
        for bit in range(nbits):
            self.specials += \
                Instance("TRELLIS_IO",
                    p_DIR="BIDIR",
                    i_B=io[bit],
                    i_I=o[bit],
                    o_O=i[bit],
                    i_T=~oe,
                )

class LatticeECPXTrellisTristate(Module):
    @staticmethod
    def lower(dr):
        return LatticeECPXTrellisTristateImpl(dr.target, dr.o, dr.oe, dr.i)

lattice_ecpx_trellis_special_overrides = {
    AsyncResetSynchronizer: LatticeECPXAsyncResetSynchronizer,
    Tristate:               LatticeECPXTrellisTristate,
    DDROutput:              LatticeECPXDDROutput
}


class LatticeiCE40AsyncResetSynchronizerImpl(Module):
    def __init__(self, cd, async_reset):
        rst1 = Signal()
        self.specials += [
            Instance("SB_DFFS", i_D=0, i_S=async_reset,
                     i_C=cd.clk, o_Q=rst1),
            Instance("SB_DFFS", i_D=rst1, i_S=async_reset,
                     i_C=cd.clk, o_Q=cd.rst)
        ]


class LatticeiCE40AsyncResetSynchronizer:
    @staticmethod
    def lower(dr):
        return LatticeiCE40AsyncResetSynchronizerImpl(dr.cd, dr.async_reset)


class LatticeiCE40TristateImpl(Module):
    def __init__(self, io, o, oe, i):
        nbits, sign = value_bits_sign(io)
        if nbits == 1:
            # If `io` is an expression like `port[x]`, it is not legal to index further
            # into it if it is only 1 bit wide.
            self.specials += \
                Instance("SB_IO",
                    p_PIN_TYPE=C(0b101001, 6),
                    io_PACKAGE_PIN=io,
                    i_OUTPUT_ENABLE=oe,
                    i_D_OUT_0=o,
                    o_D_IN_0=i,
                )
        else:
            for bit in range(nbits):
                self.specials += \
                    Instance("SB_IO",
                        p_PIN_TYPE=C(0b101001, 6),
                        io_PACKAGE_PIN=io[bit],
                        i_OUTPUT_ENABLE=oe,
                        i_D_OUT_0=o[bit],
                        o_D_IN_0=i[bit],
                    )


class LatticeiCE40Tristate(Module):
    @staticmethod
    def lower(dr):
        return LatticeiCE40TristateImpl(dr.target, dr.o, dr.oe, dr.i)


class LatticeiCE40DifferentialOutputImpl(Module):
    def __init__(self, i, o_p, o_n):
        self.specials += Instance("SB_IO",
                                  p_PIN_TYPE=C(0b011000, 6),
                                  p_IO_STANDARD="SB_LVCMOS",
                                  io_PACKAGE_PIN=o_p,
                                  i_D_OUT_0=i)

        self.specials += Instance("SB_IO",
                                  p_PIN_TYPE=C(0b011000, 6),
                                  p_IO_STANDARD="SB_LVCMOS",
                                  io_PACKAGE_PIN=o_n,
                                  i_D_OUT_0=~i)


class LatticeiCE40DifferentialOutput:
    @staticmethod
    def lower(dr):
        return LatticeiCE40DifferentialOutputImpl(dr.i, dr.o_p, dr.o_n)


# For iCE40 HX/LP devices, 'IO[Bank]_[No]B' is the positive side of the pair.
# For iCE40 UP devices, 'IO[Bank]_[No]a' is the positive side of the pair.
# The SB_IO primitive expects the 'B' pin for HX/LP devices
# and the 'a' pin for UP devices.
# The output of SB_IO has the polarity of the passed pin.
# See https://github.com/m-labs/migen/pull/181
class LatticeiCE40DifferentialInputImpl(Module):
    def __init__(self, i_p, i_n, o):
        self.specials += Instance("SB_IO",
                                  p_PIN_TYPE=C(0b000001, 6),  # simple input pin
                                  p_IO_STANDARD="SB_LVDS_INPUT",
                                  io_PACKAGE_PIN=i_p,
                                  o_D_IN_0=o)


class LatticeiCE40DifferentialInput:
    @staticmethod
    def lower(dr):
        return LatticeiCE40DifferentialInputImpl(dr.i_p, dr.i_n, dr.o)


lattice_ice40_special_overrides = {
    AsyncResetSynchronizer: LatticeiCE40AsyncResetSynchronizer,
    Tristate:               LatticeiCE40Tristate,
    DifferentialOutput:     LatticeiCE40DifferentialOutput,
    DifferentialInput:      LatticeiCE40DifferentialInput,
}



# This file is Copyright (c) 2013 Florent Kermarrec <florent@enjoy-digital.fr>
# License: BSD

import os
import subprocess

from migen.fhdl.structure import _Fragment

from migen.build.generic_platform import Pins, IOStandard, Misc
from migen.build import tools


def _format_constraint(c, signame, fmt_r):
    if isinstance(c, Pins):
        return "set_location_assignment -comment \"{name}\" " \
            "-to {signame} Pin_{pin}".format(
                signame=signame,
                name=fmt_r,
                pin=c.identifiers[0])
    elif isinstance(c, IOStandard):
        return "set_instance_assignment -name io_standard " \
            "-comment \"{name}\" \"{std}\" -to {signame}".format(
                signame=signame,
                name=fmt_r,
                std=c.name)
    elif isinstance(c, Misc):
        if not isinstance(c.misc, str) and len(c.misc) == 2:
            return "set_instance_assignment -comment \"{name}\" " \
                "-name {misc[0]} \"{misc[1]}\" -to {signame}".format(
                    signame=signame,
                    name=fmt_r,
                    misc=c.misc)
        else:
            return "set_instance_assignment -comment \"{name}\" " \
                "-name {misc} " \
                "-to {signame}".format(
                    signame=signame,
                    name=fmt_r,
                    misc=c.misc)


def _format_qsf(signame, pin, others, resname):
    fmt_r = "{}:{}".format(*resname[:2])
    if resname[2] is not None:
        fmt_r += "." + resname[2]

    fmt_c = [_format_constraint(c, signame, fmt_r) for c in
             ([Pins(pin)] + others)]

    return '\n'.join(fmt_c)


def _build_qsf(named_sc, named_pc):
    lines = []
    for sig, pins, others, resname in named_sc:
        if len(pins) > 1:
            for i, p in enumerate(pins):
                lines.append(
                    _format_qsf("{}[{}]".format(sig, i), p, others, resname))
        else:
            lines.append(_format_qsf(sig, pins[0], others, resname))

    if named_pc:
        lines.append("")
        lines.append("\n\n".join(named_pc))

    lines.append("set_global_assignment -name top_level_entity top")
    return "\n".join(lines)


def _build_files(device, sources, vincpaths, named_sc, named_pc, build_name):
    lines = []
    for filename, language, library in sources:
        # Enforce use of SystemVerilog
        # (Quartus does not support global parameters in Verilog)
        if language == "verilog":
            language = "systemverilog"
        lines.append(
            "set_global_assignment -name {lang}_FILE {path} "
            "-library {lib}".format(
                lang=language.upper(),
                path=filename.replace("\\", "/"),
                lib=library))

    for path in vincpaths:
        lines.append("set_global_assignment -name SEARCH_PATH {}".format(
            path.replace("\\", "/")))

    lines.append(_build_qsf(named_sc, named_pc))
    lines.append("set_global_assignment -name DEVICE {}".format(device))
    tools.write_to_file("{}.qsf".format(build_name), "\n".join(lines))


def _run_quartus(build_name, create_rbf):
    build_script_contents = """# Autogenerated by Migen

set -e

quartus_map --read_settings_files=on --write_settings_files=off {build_name} -c {build_name}
quartus_fit --read_settings_files=off --write_settings_files=off {build_name} -c {build_name}
quartus_asm --read_settings_files=off --write_settings_files=off {build_name} -c {build_name}
quartus_sta {build_name} -c {build_name}"""

    if create_rbf:
        build_script_contents +="""
if [ -f "{build_name}.sof" ]
then
    quartus_cpf -c {build_name}.sof {build_name}.rbf
fi

"""
    build_script_contents = build_script_contents.format(build_name=build_name)  # noqa
    build_script_file = "build_" + build_name + ".sh"
    tools.write_to_file(build_script_file,
                        build_script_contents,
                        force_unix=True)

    if subprocess.call(["bash", build_script_file]):
        raise OSError("Subprocess failed")


class AlteraQuartusToolchain:
    def build(self, platform, fragment, build_dir="build", build_name="top",
              run=True, **kwargs):
        cwd = os.getcwd()
        os.makedirs(build_dir, exist_ok=True)
        os.chdir(build_dir)

        if not isinstance(fragment, _Fragment):
            fragment = fragment.get_fragment()
        platform.finalize(fragment)

        v_output = platform.get_verilog(fragment, name=build_name, **kwargs)
        named_sc, named_pc = platform.resolve_signals(v_output.ns)
        v_file = build_name + ".v"
        v_output.write(v_file)
        sources = platform.sources | {(v_file, "verilog", "work")}
        _build_files(platform.device,
                     sources,
                     platform.verilog_include_paths,
                     named_sc,
                     named_pc,
                     build_name)
        if run:
            _run_quartus(build_name, platform.create_rbf)

        os.chdir(cwd)

        return v_output.ns

    def add_period_constraint(self, platform, clk, period):
        # TODO: handle differential clk
        platform.add_platform_command(
            "set_global_assignment -name duty_cycle 50 -section_id {clk}",
            clk=clk)
        platform.add_platform_command(
            "set_global_assignment -name fmax_requirement \"{freq} MHz\" "
            "-section_id {clk}".format(freq=(1. / period) * 1000,
                                       clk="{clk}"),
            clk=clk)



from migen.build.generic_platform import *
from migen.build.altera import AlteraPlatform
from migen.build.altera.programmer import USBBlaster


_io = [
    ("clk12", 0, Pins("H6"), IOStandard("3.3-V LVTTL")),

    ("user_led", 0, Pins("A8"), IOStandard("3.3-V LVTTL")),
    ("user_led", 1, Pins("A9"), IOStandard("3.3-V LVTTL")),
    ("user_led", 2, Pins("A11"), IOStandard("3.3-V LVTTL")),
    ("user_led", 3, Pins("A10"), IOStandard("3.3-V LVTTL")),
    ("user_led", 4, Pins("B10"), IOStandard("3.3-V LVTTL")),
    ("user_led", 5, Pins("C9"), IOStandard("3.3-V LVTTL")),
    ("user_led", 6, Pins("C10"), IOStandard("3.3-V LVTTL")),
    ("user_led", 7, Pins("D8"), IOStandard("3.3-V LVTTL")),

    ("sw", 0, Pins("E6"), IOStandard("3.3-V LVTTL")),

    ("serial", 0,
        Subsignal("tx", Pins("B4"), IOStandard("3.3-V LVTTL")),
        Subsignal("rx", Pins("A4"), IOStandard("3.3-V LVTTL")),
        Subsignal("rts", Pins("A6"), IOStandard("3.3-V LVTTL")),
        Subsignal("cts", Pins("B5"), IOStandard("3.3-V LVTTL")),
        Subsignal("dtr", Pins("A7"), IOStandard("3.3-V LVTTL")),
        Subsignal("dsr", Pins("B6"), IOStandard("3.3-V LVTTL"))
    ),

    ("sdram_clock", 0, Pins("M9"), IOStandard("3.3-V LVTTL")),
    ("sdram", 0,
        Subsignal("a", Pins("K6 M5 N5 J8 N10 M11 N9 L10 M13 N8 N4 M10 L11 M12")),
        Subsignal("ba", Pins("N6 K8")),
        Subsignal("cs_n", Pins("M4")),
        Subsignal("cke", Pins("M8")),
        Subsignal("ras_n", Pins("M7")),
        Subsignal("cas_n", Pins("N7")),
        Subsignal("we_n", Pins("K7")),
        Subsignal("dq", Pins("D11 G10 F10 F9 E10 D9 G9 F8 F13 E12 E13 D12 C12 B12 B13 A12")),
        Subsignal("dm", Pins("E9 F12")),
        IOStandard("3.3-V LVTTL")
    ),

    ("accelerometer", 0,
        Subsignal("int1", Pins("J5")),
        Subsignal("int1", Pins("L4")),
        Subsignal("mosi", Pins("J7")),
        Subsignal("miso", Pins("K5")),
        Subsignal("clk", Pins("J6")),
        Subsignal("cs_n", Pins("L5")),
        IOStandard("3.3-V LVTTL")
    ),

    ("spiflash", 0,
        Subsignal("cs_n", Pins("B3")),
        Subsignal("clk", Pins("A3")),
        Subsignal("dio0", Pins("A2")),
        Subsignal("dio1", Pins("B2")),
        Subsignal("dio2", Pins("C4")),
        Subsignal("dio3", Pins("B9")),
        IOStandard("3.3-V LVTTL")
    ),

    # J1 connector
    ("gpio_0", 0,
        Pins("D3 E1 C2 C1 D1 E3 F1 E4 H8 K10 H5 H4 J1 J2"),
        IOStandard("3.3-V LVTTL")
    ),

    # J2 connector
    ("gpio_1", 0,
        Pins("L12 J12 J13 K11 K12 J10 H10 H13 G12 B11 G13"),
        IOStandard("3.3-V LVTTL")
    ),

    ("pmod", 0,
        Subsignal("d", Pins("F15 F16 C17 C18 F14 G14 D17 D18")),
        IOStandard("3.3-V LVTTL")
    )
]


class Platform(AlteraPlatform):
    default_clk_name = "clk12"
    default_clk_period = 83.333
    create_rbf = False

    def __init__(self):
        AlteraPlatform.__init__(self, "10M08SAU169C8G", _io)

    def create_programmer(self):
        return USBBlaster()



import unittest
import random

from migen import *
from migen.genlib.misc import BitSlip


class BitSlipModel:
    def __init__(self, data_width, latency):
        self.data_width = data_width
        self.latency = latency

    def simulate(self, bitslip, sequence):
        # prepare sequence for simulation
        s = [0]*self.latency
        for d in sequence:
            s.append(d)
        # simulate bitslip
        r = []
        for i in range(len(s)-1):
            v = (s[i+1] << self.data_width) | s[i]
            v = v >> bitslip
            v &= 2**self.data_width-1
            r.append(v)
        return r


def main_generator(dut):
    dut.o_sequence = []
    yield dut.value.eq(dut.bitslip)
    for i, data in enumerate(dut.i_sequence):
        yield dut.i.eq(data)
        dut.o_sequence.append((yield dut.o))
        yield


class TestBitSlip(unittest.TestCase):
    def bitslip_test(self, data_width, length=128):
        prng = random.Random(42)
        sequence = [prng.randrange(2**data_width) for i in range(length)]

        for i in range(data_width):
            dut = BitSlip(data_width)
            dut.bitslip = i
            dut.i_sequence = sequence
            run_simulation(dut, main_generator(dut))

            model = BitSlipModel(data_width, 4)
            m_sequence = model.simulate(i, sequence)

            self.assertEqual(dut.o_sequence, m_sequence[:len(dut.o_sequence)])

    def test_bitslip_4b(self):
        self.bitslip_test(4)

    def test_bitslip_8b(self):
        self.bitslip_test(8)

    def test_bitslip_16b(self):
        self.bitslip_test(16)

    def test_bitslip_32b(self):
        self.bitslip_test(32)

    def test_bitslip_64b(self):
        self.bitslip_test(64)

    def test_bitslip_128b(self):
        self.bitslip_test(128)



import os

from migen.build.generic_programmer import GenericProgrammer
from migen.build.xilinx.programmer import _create_xsvf

try:
   import fl
except ImportError:
   import fpgalink3 as fl

fl.flInitialise(0)


class FPGALink(GenericProgrammer):
    """Using the fpgalink library from makestuff

    You will need fpgalink library installed from
    https://github.com/makestuff/libfpgalink
    """

    needs_bitreverse = False

    def __init__(self, initial_vidpid=None, pin_cfg="D0D2D3D4",
                 fpgalink_vidpid="1D50:602B:0002", flash_proxy_basename=None):
        """
        Parameters
        ----------
        initial_vidpid : string
            The USB vendor and product id of the device before fpgalink
            firmware is loaded onto the device.

            Format is vid:pid as 4 digit hex numbers.

        pin_cfg : string
            FPGALink pin configuration string describing how the JTAG interface
            is hooked up to the programmer.

        fpgalink_vidpid : string
            The USB vendor, product and device id of the device after the
            fpgalink firmware is loaded onto the device.

            Format is vid:pid:did as 4 digit hex numbers.
            Defaults to 1D50:602B:0002 which is the makestuff FPGALink device.
        """
        GenericProgrammer.__init__(self, flash_proxy_basename)
        self.initial_vidpid = initial_vidpid
        self.fpgalink_vidpid = fpgalink_vidpid
        self.pin_cfg = pin_cfg

    def open_device(self):
        ivp = self.initial_vidpid
        vp = self.fpgalink_vidpid

        print("Attempting to open connection to FPGALink device", vp, "...")
        try:
            handle = fl.flOpen(self.fpgalink_vidpid)
        except fl.FLException as ex:
            if not ivp:
                raise FLException(
                    "Could not open FPGALink device at {0} and"
                    " no initial VID:PID was supplied".format(vp))

            print("Loading firmware into %s..." % ivp)
            fl.flLoadStandardFirmware(ivp, vp)

            print("Awaiting renumeration...")
            if not fl.flAwaitDevice(vp, 600):
                raise fl.FLException(
                    "FPGALink device did not renumerate properly"
                    " as {0}".format(vp))

            print("Attempting to open connection to FPGALink device", vp,
                  "again...")
            handle = fl.flOpen(vp)

        # Only Nero capable hardware support doing programming.
        assert fl.flIsNeroCapable(handle)
        print("Cable connection opened.")
        return handle

    def load_bitstream(self, bitstream_file):
        n = 27

        xsvf_file = os.path.splitext(bitstream_file)[0]+'.xsvf'
        print("\nGenerating xsvf formatted bitstream")
        print("="*n)
        if os.path.exists(xsvf_file):
            os.unlink(xsvf_file)
        _create_xsvf(bitstream_file, xsvf_file)
        print("\n"+"="*n+"\n")

        print("Programming %s to device." % xsvf_file)
        print("="*n)
        handle = self.open_device()
        print("Programming device...")
        fl.flProgram(handle, "J:"+self.pin_cfg, progFile=xsvf_file)
        print("Programming successful!")
        print("="*n+"\n")
        fl.flClose(handle)

    def flash(self, address, data_file):
        raise NotImplementedError("Not supported yet.")



from migen.fhdl.structure import *
from migen.fhdl.module import Module
from migen.fhdl.specials import Special


class DifferentialInput(Special):
    def __init__(self, i_p, i_n, o):
        Special.__init__(self)
        self.i_p = wrap(i_p)
        self.i_n = wrap(i_n)
        self.o = wrap(o)

    def iter_expressions(self):
        yield self, "i_p", SPECIAL_INPUT
        yield self, "i_n", SPECIAL_INPUT
        yield self, "o", SPECIAL_OUTPUT

    @staticmethod
    def lower(dr):
        raise NotImplementedError("Attempted to use a differential input, but platform does not support them")


class DifferentialOutput(Special):
    def __init__(self, i, o_p, o_n):
        Special.__init__(self)
        self.i = wrap(i)
        self.o_p = wrap(o_p)
        self.o_n = wrap(o_n)

    def iter_expressions(self):
        yield self, "i", SPECIAL_INPUT
        yield self, "o_p", SPECIAL_OUTPUT
        yield self, "o_n", SPECIAL_OUTPUT

    @staticmethod
    def lower(dr):
        raise NotImplementedError("Attempted to use a differential output, but platform does not support them")


class CRG(Module):
    """ Clock and Reset Generator """

    def __init__(self, clk, rst=0):
        self.clock_domains.cd_sys = ClockDomain()
        self.clock_domains.cd_por = ClockDomain(reset_less=True)

        if hasattr(clk, "p"):
            clk_se = Signal()
            self.specials += DifferentialInput(clk.p, clk.n, clk_se)
            clk = clk_se

        # Power on Reset (vendor agnostic)
        int_rst = Signal(reset=1)
        self.sync.por += int_rst.eq(rst)
        self.comb += [
            self.cd_sys.clk.eq(clk),
            self.cd_por.clk.eq(clk),
            self.cd_sys.rst.eq(int_rst)
        ]


class DDRInput(Special):
    def __init__(self, i, o1, o2, clk=ClockSignal()):
        Special.__init__(self)
        self.i = wrap(i)
        self.o1 = wrap(o1)
        self.o2 = wrap(o2)
        self.clk = wrap(clk)

    def iter_expressions(self):
        yield self, "i", SPECIAL_INPUT
        yield self, "o1", SPECIAL_OUTPUT
        yield self, "o2", SPECIAL_OUTPUT
        yield self, "clk", SPECIAL_INPUT

    @staticmethod
    def lower(dr):
        raise NotImplementedError("Attempted to use a DDR input, but platform does not support them")


class DDROutput(Special):
    def __init__(self, i1, i2, o, clk=ClockSignal()):
        Special.__init__(self)
        self.i1 = i1
        self.i2 = i2
        self.o = o
        self.clk = clk

    def iter_expressions(self):
        yield self, "i1", SPECIAL_INPUT
        yield self, "i2", SPECIAL_INPUT
        yield self, "o", SPECIAL_OUTPUT
        yield self, "clk", SPECIAL_INPUT

    @staticmethod
    def lower(dr):
        raise NotImplementedError("Attempted to use a DDR output, but platform does not support them")




import unittest

from migen import *
from migen.genlib.divider import Divider
from migen.test.support import SimCase


class DivisionCase(SimCase, unittest.TestCase):
    class TestBench(Module):
        def __init__(self):
            self.submodules.dut = Divider(4)

    def test_division(self):
        def gen():
            for dividend in range(16):
                for divisor in range(1, 16):
                    with self.subTest(dividend=dividend, divisor=divisor):
                        yield self.tb.dut.dividend_i.eq(dividend)
                        yield self.tb.dut.divisor_i.eq(divisor)
                        yield self.tb.dut.start_i.eq(1)
                        yield
                        yield self.tb.dut.start_i.eq(0)
                        yield
                        while not (yield self.tb.dut.ready_o):
                            yield
                        self.assertEqual((yield self.tb.dut.quotient_o), dividend//divisor)
                        self.assertEqual((yield self.tb.dut.remainder_o), dividend%divisor)
        self.run_with(gen())



import unittest
from random import randrange

from migen import *
from migen.genlib.sort import *

from migen.test.support import SimCase


class BitonicCase(SimCase, unittest.TestCase):
    class TestBench(Module):
        def __init__(self):
            self.submodules.dut = BitonicSort(8, 4, ascending=True)

    def test_sizes(self):
        self.assertEqual(len(self.tb.dut.i), 8)
        self.assertEqual(len(self.tb.dut.o), 8)
        for i in range(8):
            self.assertEqual(len(self.tb.dut.i[i]), 4)
            self.assertEqual(len(self.tb.dut.o[i]), 4)

    def test_sort(self):
        def gen():
            for repeat in range(20):
                for i in self.tb.dut.i:
                    yield i.eq(randrange(1<<len(i)))
                yield
                self.assertEqual(sorted((yield self.tb.dut.i)),
                                 (yield self.tb.dut.o))
        self.run_with(gen())



from migen.build.generic_platform import *
from migen.build.lattice import LatticePlatform
from migen.build.lattice.programmer import IceStormProgrammer


_io = [
    ("user_led", 0, Pins("B5"), IOStandard("LVCMOS33")),
    ("user_led", 1, Pins("B4"), IOStandard("LVCMOS33")),
    ("user_led", 2, Pins("A2"), IOStandard("LVCMOS33")),
    ("user_led", 3, Pins("A1"), IOStandard("LVCMOS33")),
    ("user_led", 4, Pins("C5"), IOStandard("LVCMOS33")),
    ("user_led", 5, Pins("C4"), IOStandard("LVCMOS33")),
    ("user_led", 6, Pins("B3"), IOStandard("LVCMOS33")),
    ("user_led", 7, Pins("C3"), IOStandard("LVCMOS33")),

    ("serial", 0,
        Subsignal("rx", Pins("B10")),
        Subsignal("tx", Pins("B12"), Misc("PULLUP")),
        Subsignal("rts", Pins("B13"), Misc("PULLUP")),
        Subsignal("cts", Pins("A15"), Misc("PULLUP")),
        Subsignal("dtr", Pins("A16"), Misc("PULLUP")),
        Subsignal("dsr", Pins("B14"), Misc("PULLUP")),
        Subsignal("dcd", Pins("B15"), Misc("PULLUP")),
        IOStandard("LVCMOS33"),
    ),
    
    ("spiflash", 0,
        Subsignal("cs_n", Pins("R12"), IOStandard("LVCMOS33")),
        Subsignal("clk", Pins("R11"), IOStandard("LVCMOS33")),
        Subsignal("mosi", Pins("P12"), IOStandard("LVCMOS33")),
        Subsignal("miso", Pins("P11"), IOStandard("LVCMOS33")),
    ),

    ("clk12", 0, Pins("J3"), IOStandard("LVCMOS33"))
]


class Platform(LatticePlatform):
    default_clk_name = "clk12"
    default_clk_period = 83.333

    def __init__(self):
        LatticePlatform.__init__(self, "ice40-hx8k-ct256", _io,
                                 toolchain="icestorm")

    def create_programmer(self):
        return IceStormProgrammer()



from migen.fhdl.structure import *
from migen.fhdl.module import *
from migen.fhdl.specials import *
from migen.fhdl.bitcontainer import *
from migen.fhdl.decorators import *
from migen.fhdl.simplify import *

from migen.sim import *

from migen.genlib.record import *
from migen.genlib.fsm import *



from migen.build.generic_platform import *
from migen.build.lattice import LatticePlatform
from migen.build.lattice.programmer import TinyFpgaBProgrammer

_io = [
    ("usb", 0,
        Subsignal("d_p", Pins("A3")),
        Subsignal("d_n", Pins("A4")),
        IOStandard("LVCMOS33")
    ),

    ("spiflash", 0,
        Subsignal("cs_n", Pins("F7"), IOStandard("LVCMOS33")),
        Subsignal("clk", Pins("G7"), IOStandard("LVCMOS33")),
        Subsignal("mosi", Pins("G6"), IOStandard("LVCMOS33")),
        Subsignal("miso", Pins("H7"), IOStandard("LVCMOS33"))
    ),

    ("clk16", 0, Pins("B4"), IOStandard("LVCMOS33"))
]

_connectors = [
    # B2-J1, Pins 4-13
    # D9-C9, Pins 18-19, Pins 21-24
    # E8, Pin 20 (Input only)
    ("GPIO", "B2 A2 A1 B1 C1 D1 E1 G1 H1 J1 D9 C9 A9 A8 A7 A6"),
    ("GBIN", "E8")
]


# Default peripherals
serial = [
    ("serial", 0,
        Subsignal("tx", Pins("GPIO:0")),
        Subsignal("rx", Pins("GPIO:1")),
        IOStandard("LVCMOS33")
    )
]


class Platform(LatticePlatform):
    default_clk_name = "clk16"
    default_clk_period = 62.5

    def __init__(self):
        LatticePlatform.__init__(self, "ice40-lp8k-cm81", _io, _connectors,
                                 toolchain="icestorm")

    def create_programmer(self):
        return TinyFpgaBProgrammer()



# This file is Copyright (c) 2013 Florent Kermarrec <florent@enjoy-digital.fr>
# License: BSD

from migen.build.generic_platform import *
from migen.build.lattice import LatticePlatform
from migen.build.lattice.programmer import LatticeProgrammer


_io = [
    ("clk100", 0, Pins("L5"), IOStandard("LVDS25")),
    ("rst_n", 0, Pins("A21"), IOStandard("LVCMOS33")),

    ("user_led", 0, Pins("Y20"), IOStandard("LVCMOS33")),
    ("user_led", 1, Pins("AA21"), IOStandard("LVCMOS33")),
    ("user_led", 2, Pins("U18"), IOStandard("LVCMOS33")),
    ("user_led", 3, Pins("U19"), IOStandard("LVCMOS33")),
    ("user_led", 4, Pins("W19"), IOStandard("LVCMOS33")),
    ("user_led", 5, Pins("V19"), IOStandard("LVCMOS33")),
    ("user_led", 6, Pins("AB20"), IOStandard("LVCMOS33")),
    ("user_led", 7, Pins("AA20"), IOStandard("LVCMOS33")),

    ("user_dip_btn", 0, Pins("J7"), IOStandard("LVCMOS15")),
    ("user_dip_btn", 1, Pins("J6"), IOStandard("LVCMOS15")),
    ("user_dip_btn", 2, Pins("H2"), IOStandard("LVCMOS15")),
    ("user_dip_btn", 3, Pins("H3"), IOStandard("LVCMOS15")),
    ("user_dip_btn", 4, Pins("J3"), IOStandard("LVCMOS15")),
    ("user_dip_btn", 5, Pins("K3"), IOStandard("LVCMOS15")),
    ("user_dip_btn", 6, Pins("J2"), IOStandard("LVCMOS15")),
    ("user_dip_btn", 7, Pins("J1"), IOStandard("LVCMOS15")),

    ("serial", 0,
        Subsignal("tx", Pins("B11"), IOStandard("LVCMOS33")),  # X4 IO0
        Subsignal("rx", Pins("B12"), IOStandard("LVCMOS33")),  # X4 IO1
    ),

    ("eth_clocks", 0,
        Subsignal("tx", Pins("C12")),
        Subsignal("gtx", Pins("M2")),
        Subsignal("rx", Pins("L4")),
        IOStandard("LVCMOS33")
    ),
    ("eth", 0,
        Subsignal("rst_n", Pins("L3")),
        Subsignal("mdio", Pins("L2")),
        Subsignal("mdc", Pins("V4")),
        Subsignal("rx_dv", Pins("M1")),
        Subsignal("rx_er", Pins("M4")),
        Subsignal("rx_data", Pins("M5 N1 N6 P6 T2 R2 P5 P3")),
        Subsignal("tx_en", Pins("V3")),
        Subsignal("tx_data", Pins("V1 U1 R3 P1 N5 N3 N4 N2")),
        Subsignal("col", Pins("R1")),
        Subsignal("crs", Pins("P4")),
        IOStandard("LVCMOS33")
    ),

    ("eth_clocks", 1,
        Subsignal("tx", Pins("M21")),
        Subsignal("gtx", Pins("M19")),
        Subsignal("rx", Pins("N19")),
        IOStandard("LVCMOS33")
    ),
    ("eth", 1,
        Subsignal("rst_n", Pins("R21")),
        Subsignal("mdio", Pins("U16")),
        Subsignal("mdc", Pins("Y18")),
        Subsignal("rx_dv", Pins("U15")),
        Subsignal("rx_er", Pins("V20")),
        Subsignal("rx_data", Pins("AB17 AA17 R19 V21 T17 R18 W21 Y21")),
        Subsignal("tx_en", Pins("V22")),
        Subsignal("tx_data", Pins("W22 R16 P17 Y22 T21 U22 P20 U20")),
        Subsignal("col", Pins("N18")),
        Subsignal("crs", Pins("P19")),
        IOStandard("LVCMOS33")
    ),
]


class Platform(LatticePlatform):
    default_clk_name = "clk100"
    default_clk_period = 10

    def __init__(self, **kwargs):
        LatticePlatform.__init__(self, "LFE3-35EA-6FN484C", _io, **kwargs)

    def do_finalize(self, fragment):
        LatticePlatform.do_finalize(self, fragment)
        try:
            self.add_period_constraint(self.lookup_request("eth_clocks", 0).rx, 8.0)
        except ConstraintError:
            pass
        try:
            self.add_period_constraint(self.lookup_request("eth_clocks", 1).rx, 8.0)
        except ConstraintError:
            pass

    def create_programmer(self):
        return LatticeProgrammer()



import os
import sys
try:
    import colorama
    colorama.init()  # install escape sequence translation on Windows
    _have_colorama = True
except ImportError:
    _have_colorama = False

from migen.fhdl.structure import *
from migen.fhdl.specials import Instance
from migen.fhdl.module import Module
from migen.genlib.cdc import *
from migen.genlib.resetsync import AsyncResetSynchronizer
from migen.genlib.io import *

from migen.build import tools


colors = []
if _have_colorama:
    colors += [
        ("^ERROR:.*$", colorama.Fore.RED + colorama.Style.BRIGHT +
         r"\g<0>" + colorama.Style.RESET_ALL),
        ("^CRITICAL WARNING:.*$", colorama.Fore.RED +
         r"\g<0>" + colorama.Style.RESET_ALL),
        ("^WARNING:.*$", colorama.Fore.YELLOW +
         r"\g<0>" + colorama.Style.RESET_ALL),
        ("^INFO:.*$", colorama.Fore.GREEN +
         r"\g<0>" + colorama.Style.RESET_ALL),
    ]


class XilinxMultiRegImpl(MultiRegImpl):
    def __init__(self, *args, **kwargs):
        MultiRegImpl.__init__(self, *args, **kwargs)
        i = self.i
        if not hasattr(i, "attr"):
            i0, i = i, Signal()
            self.comb += i.eq(i0)
        self.regs[0].attr.add("mr_ff")
        for r in self.regs:
            r.attr.add("async_reg")
            r.attr.add("no_shreg_extract")


class XilinxMultiReg:
    @staticmethod
    def lower(dr):
        return XilinxMultiRegImpl(dr.i, dr.o, dr.odomain, dr.n)


class XilinxAsyncResetSynchronizerImpl(Module):
    def __init__(self, cd, async_reset):
        if not hasattr(async_reset, "attr"):
            i, async_reset = async_reset, Signal()
            self.comb += async_reset.eq(i)
        rst_meta = Signal()
        self.specials += [
            Instance("FDPE", p_INIT=1, i_D=0, i_PRE=async_reset,
                i_CE=1, i_C=cd.clk, o_Q=rst_meta,
                attr={"async_reg", "ars_ff1"}),
            Instance("FDPE", p_INIT=1, i_D=rst_meta, i_PRE=async_reset,
                i_CE=1, i_C=cd.clk, o_Q=cd.rst,
                attr={"async_reg", "ars_ff2"})
        ]


class XilinxAsyncResetSynchronizer:
    @staticmethod
    def lower(dr):
        return XilinxAsyncResetSynchronizerImpl(dr.cd, dr.async_reset)


class XilinxDifferentialInputImpl(Module):
    def __init__(self, i_p, i_n, o):
        self.specials += Instance("IBUFDS", i_I=i_p, i_IB=i_n, o_O=o)


class XilinxDifferentialInput:
    @staticmethod
    def lower(dr):
        return XilinxDifferentialInputImpl(dr.i_p, dr.i_n, dr.o)


class XilinxDifferentialOutputImpl(Module):
    def __init__(self, i, o_p, o_n):
        self.specials += Instance("OBUFDS", i_I=i, o_O=o_p, o_OB=o_n)


class XilinxDifferentialOutput:
    @staticmethod
    def lower(dr):
        return XilinxDifferentialOutputImpl(dr.i, dr.o_p, dr.o_n)


xilinx_special_overrides = {
    MultiReg:               XilinxMultiReg,
    AsyncResetSynchronizer: XilinxAsyncResetSynchronizer,
    DifferentialInput:      XilinxDifferentialInput,
    DifferentialOutput:     XilinxDifferentialOutput
}


class XilinxDDROutputImplS6(Module):
    def __init__(self, i1, i2, o, clk):
        self.specials += Instance("ODDR2",
                p_DDR_ALIGNMENT="C0", p_INIT=0, p_SRTYPE="ASYNC",
                i_C0=clk, i_C1=~clk, i_CE=1, i_S=0, i_R=0,
                i_D0=i1, i_D1=i2, o_Q=o,
        )


class XilinxDDROutputS6:
    @staticmethod
    def lower(dr):
        return XilinxDDROutputImplS6(dr.i1, dr.i2, dr.o, dr.clk)


xilinx_s6_special_overrides = {
    DDROutput:              XilinxDDROutputS6
}


class XilinxDDROutputImplS7(Module):
    def __init__(self, i1, i2, o, clk):
        self.specials += Instance("ODDR",
                p_DDR_CLK_EDGE="SAME_EDGE",
                i_C=clk, i_CE=1, i_S=0, i_R=0,
                i_D1=i1, i_D2=i2, o_Q=o,
        )


class XilinxDDROutputS7:
    @staticmethod
    def lower(dr):
        return XilinxDDROutputImplS7(dr.i1, dr.i2, dr.o, dr.clk)


class XilinxDDRInputImplS7(Module):
    def __init__(self, i, o1, o2, clk):
        self.specials += Instance("IDDR",
                p_DDR_CLK_EDGE="SAME_EDGE",
                i_C=clk, i_CE=1, i_S=0, i_R=0,
                i_D=i, o_Q1=o1, o_Q2=o2,
        )


class XilinxDDRInputS7:
    @staticmethod
    def lower(dr):
        return XilinxDDRInputImplS7(dr.i, dr.o1, dr.o2, dr.clk)


xilinx_s7_special_overrides = {
    DDROutput:              XilinxDDROutputS7,
    DDRInput:               XilinxDDRInputS7
}


class XilinxDDROutputImplKU(Module):
    def __init__(self, i1, i2, o, clk):
        self.specials += Instance("ODDRE1",
                i_C=clk, i_SR=0,
                i_D1=i1, i_D2=i2, o_Q=o,
        )


class XilinxDDROutputKU:
    @staticmethod
    def lower(dr):
        return XilinxDDROutputImplKU(dr.i1, dr.i2, dr.o, dr.clk)


class XilinxDDRInputImplKU(Module):
    def __init__(self, i, o1, o2, clk):
        self.specials += Instance("IDDRE1",
            p_DDR_CLK_EDGE="SAME_EDGE_PIPELINED",
            p_IS_C_INVERTED=0,
            p_IS_CB_INVERTED=1,
            i_D=i,
            o_Q1=o1, o_Q2=o2,
            i_C=clk, i_CB=clk,
            i_R=0
        )


class XilinxDDRInputKU:
    @staticmethod
    def lower(dr):
        return XilinxDDRInputImplKU(dr.i, dr.o1, dr.o2, dr.clk)


xilinx_ku_special_overrides = {
    DDROutput:              XilinxDDROutputKU,
    DDRInput:               XilinxDDRInputKU
}



from migen.build.generic_platform import *
from migen.build.xilinx import XilinxPlatform


_io = [
    ("user_led", 0, Pins("J24"), IOStandard("LVCMOS33")), # sfp0_led1
    ("user_led", 1, Pins("J25"), IOStandard("LVCMOS33")), # sfp0_led2
    ("user_led", 2, Pins("L25"), IOStandard("LVCMOS33")), # sfp1_led1
    ("user_led", 3, Pins("K25"), IOStandard("LVCMOS33")), # sfp1_led2

    ("clk50", 0, Pins("N24"), IOStandard("LVCMOS33")),

    ("serial", 0,
        Subsignal("tx", Pins("K20")),
        Subsignal("rx", Pins("K22")),
        IOStandard("LVCMOS33")
    ),

    ("i2c", 0,
        Subsignal("scl", Pins("N21")),
        Subsignal("sda", Pins("M21")),
        IOStandard("LVCMOS33")
    ),

    # this is the second SPI flash (not containing the bitstream)
    # clock is shared with the bitstream flash and needs to be accessed
    # through STARTUPE3
    ("spiflash", 0,
        Subsignal("cs_n", Pins("G26")),
        Subsignal("dq", Pins("M20 L20 R21 R22")),
        IOStandard("LVCMOS33")
    ),

    ("ddram", 0,
        Subsignal("a", Pins(
            "AE17 AL17 AG16 AG17 AD16 AH14 AD15 AK15",
            "AF14 AF15 AL18 AL15 AE18 AJ15 AG14"),
            IOStandard("SSTL15_DCI")),
        Subsignal("ba", Pins("AF17 AD19 AD18"), IOStandard("SSTL15_DCI")),
        Subsignal("ras_n", Pins("AH19"), IOStandard("SSTL15_DCI")),
        Subsignal("cas_n", Pins("AK18"), IOStandard("SSTL15_DCI")),
        Subsignal("we_n", Pins("AG19"), IOStandard("SSTL15_DCI")),
        Subsignal("cs_n", Pins("AF18"), IOStandard("SSTL15_DCI")),
        Subsignal("dm", Pins("AD21 AE25 AJ21 AM21 AH26 AN26 AJ29 AL32"),
            IOStandard("SSTL15_DCI"),
            Misc("DATA_RATE=DDR")),
        Subsignal("dq", Pins(
            "AE23 AG20 AF22 AF20 AE22 AD20 AG22 AE20",
            "AJ24 AG24 AJ23 AF23 AH23 AF24 AH22 AG25",
            "AL22 AL25 AM20 AK23 AK22 AL24 AL20 AL23",
            "AM24 AN23 AN24 AP23 AP25 AN22 AP24 AM22",
            "AH28 AK26 AK28 AM27 AJ28 AH27 AK27 AM26",
            "AL30 AP29 AM30 AN28 AL29 AP28 AM29 AN27",
            "AH31 AH32 AJ34 AK31 AJ31 AJ30 AH34 AK32",
            "AN33 AP33 AM34 AP31 AM32 AN31 AL34 AN32"),
            IOStandard("SSTL15_DCI"),
            Misc("ODT=RTT_40"),
            Misc("DATA_RATE=DDR")),
        Subsignal("dqs_p", Pins("AG21 AH24 AJ20 AP20 AL27 AN29 AH33 AN34"),
            IOStandard("DIFF_SSTL15_DCI"),
            Misc("ODT=RTT_40"),
            Misc("DATA_RATE=DDR")),
        Subsignal("dqs_n", Pins("AH21 AJ25 AK20 AP21 AL28 AP30 AJ33 AP34"),
            IOStandard("DIFF_SSTL15_DCI"),
            Misc("ODT=RTT_40"),
            Misc("DATA_RATE=DDR")),
        Subsignal("clk_p", Pins("AE16"), IOStandard("DIFF_SSTL15_DCI"), Misc("DATA_RATE=DDR")),
        Subsignal("clk_n", Pins("AE15"), IOStandard("DIFF_SSTL15_DCI"), Misc("DATA_RATE=DDR")),
        Subsignal("cke", Pins("AL19"), IOStandard("SSTL15_DCI")),
        Subsignal("odt", Pins("AJ18"), IOStandard("SSTL15_DCI")),
        Subsignal("reset_n", Pins("AJ14"), IOStandard("SSTL15")),
        Misc("SLEW=FAST"),
        Misc("OUTPUT_IMPEDANCE=RDRV_40_40")
    ),

    ("gth_clk200", 0,
        Subsignal("p", Pins("T6")),
        Subsignal("n", Pins("T5"))
    ),
    ("port0", 0,
        Subsignal("txp", Pins("B6")),
        Subsignal("txn", Pins("B5")),
        Subsignal("rxp", Pins("A4")),
        Subsignal("rxn", Pins("A3"))
    ),

    ("si5324", 0,
        Subsignal("rst_n", Pins("AL13"), IOStandard("LVCMOS33")),
        Subsignal("int", Pins("AK13"), IOStandard("LVCMOS33"))
    ),
    ("cdr_clk_clean", 0,
        Subsignal("p", Pins("V6")),
        Subsignal("n", Pins("V5"))
    ),
    ("mch_fabric_d", 0,
        Subsignal("txp", Pins("AN4")),
        Subsignal("txn", Pins("AN3")),
        Subsignal("rxp", Pins("AP2")),
        Subsignal("rxn", Pins("AP1"))
    ),
    ("mch_fabric_d", 1,
        Subsignal("txp", Pins("AL4")),
        Subsignal("txn", Pins("AL3")),
        Subsignal("rxp", Pins("AK2")),
        Subsignal("rxn", Pins("AK1"))
    ),
    ("mch_fabric_d", 2,
        Subsignal("txp", Pins("AH6")),
        Subsignal("txn", Pins("AH5")),
        Subsignal("rxp", Pins("AH2")),
        Subsignal("rxn", Pins("AH1"))
    ),
    ("mch_fabric_d", 3,
        Subsignal("txp", Pins("AE4")),
        Subsignal("txn", Pins("AE3")),
        Subsignal("rxp", Pins("AD2")),
        Subsignal("rxn", Pins("AD1"))
    ),
    ("mch_fabric_d", 4,
        Subsignal("txp", Pins("AA4")),
        Subsignal("txn", Pins("AA3")),
        Subsignal("rxp", Pins("Y2")),
        Subsignal("rxn", Pins("Y1"))
    ),
    ("mch_fabric_d", 5,
        Subsignal("txp", Pins("U4")),
        Subsignal("txn", Pins("U3")),
        Subsignal("rxp", Pins("T2")),
        Subsignal("rxn", Pins("T1"))
    ),
    ("mch_fabric_d", 6,
        Subsignal("txp", Pins("AM6")),
        Subsignal("txn", Pins("AM5")),
        Subsignal("rxp", Pins("AM2")),
        Subsignal("rxn", Pins("AM1"))
    ),
    ("mch_fabric_d", 7,
        Subsignal("txp", Pins("AK6")),
        Subsignal("txn", Pins("AK5")),
        Subsignal("rxp", Pins("AJ4")),
        Subsignal("rxn", Pins("AJ3"))
    ),
    ("mch_fabric_d", 8,
        Subsignal("txp", Pins("AG4")),
        Subsignal("txn", Pins("AG3")),
        Subsignal("rxp", Pins("AF2")),
        Subsignal("rxn", Pins("AF1"))
    ),
    ("mch_fabric_d", 9,
        Subsignal("txp", Pins("AC4")),
        Subsignal("txn", Pins("AC3")),
        Subsignal("rxp", Pins("AB2")),
        Subsignal("rxn", Pins("AB1"))
    ),
    ("mch_fabric_d", 10,
        Subsignal("txp", Pins("W4")),
        Subsignal("txn", Pins("W3")),
        Subsignal("rxp", Pins("V2")),
        Subsignal("rxn", Pins("V1"))
    ),
    ("mch_fabric_d", 11,
        Subsignal("txp", Pins("R4")),
        Subsignal("txn", Pins("R3")),
        Subsignal("rxp", Pins("P2")),
        Subsignal("rxn", Pins("P1"))
    ),
]


# via VHDCI carrier v1.1
_connectors = [
    ("eem0", {
        "d0_cc_n": "D25",
        "d0_cc_p": "E25",
        "d1_n": "C22",
        "d1_p": "C21",
        "d2_n": "D21",
        "d2_p": "D20",
        "d3_n": "F25",
        "d3_p": "G24",
        "d4_n": "B22",
        "d4_p": "B21",
        "d5_n": "C23",
        "d5_p": "D23",
        "d6_n": "A24",
        "d6_p": "B24",
        "d7_n": "E21",
        "d7_p": "E20",
    }),

    ("eem1", {
        "d0_cc_n": "E23",
        "d0_cc_p": "E22",
        "d1_n": "A28",
        "d1_p": "A27",
        "d2_n": "F24",
        "d2_p": "F23",
        "d3_n": "A29",
        "d3_p": "B29",
        "d4_n": "B27",
        "d4_p": "C27",
        "d5_n": "A20",
        "d5_p": "B20",
        "d6_n": "C28",
        "d6_p": "D28",
        "d7_n": "D26",
        "d7_p": "E26",
    }),

    ("eem2", {
        "d0_cc_n": "C24",
        "d0_cc_p": "D24",
        "d1_n": "A25",
        "d1_p": "B25",
        "d2_n": "B26",
        "d2_p": "C26",
        "d3_n": "D29",
        "d3_p": "E28",
        "d4_n": "F20",
        "d4_p": "G20",
        "d5_n": "E27",
        "d5_p": "F27",
        "d6_n": "F22",
        "d6_p": "G22",
        "d7_n": "G21",
        "d7_p": "H21",
    }),

    ("eem3", {
        "d0_cc_n": "AB31",
        "d0_cc_p": "AB30",
        "d1_n": "Y30",
        "d1_p": "W30",
        "d2_n": "Y32",
        "d2_p": "Y31",
        "d3_n": "W31",
        "d3_p": "V31",
        "d4_n": "Y33",
        "d4_p": "W33",
        "d5_n": "W34",
        "d5_p": "V33",
        "d6_n": "AD33",
        "d6_p": "AC33",
        "d7_n": "V34",
        "d7_p": "U34",
    }),

    ("eem4", {
        "d0_cc_n": "W24",
        "d0_cc_p": "W23",
        "d1_n": "U25",
        "d1_p": "U24",
        "d2_n": "W29",
        "d2_p": "V29",
        "d3_n": "V28",
        "d3_p": "V27",
        "d4_n": "AD31",
        "d4_p": "AD30",
        "d5_n": "AG30",
        "d5_p": "AF30",
        "d6_n": "Y28",
        "d6_p": "W28",
        "d7_n": "U27",
        "d7_p": "U26",
    }),

    ("eem5", {
        "d0_cc_n": "AA25",
        "d0_cc_p": "AA24",
        "d1_n": "T23",
        "d1_p": "T22",
        "d2_n": "AD28",
        "d2_p": "AC28",
        "d3_n": "W21",
        "d3_p": "V21",
        "d4_n": "AB20",
        "d4_p": "AA20",
        "d5_n": "AF34",
        "d5_p": "AE33",
        "d6_n": "AC21",
        "d6_p": "AB21",
        "d7_n": "AB29",
        "d7_p": "AA29",
    }),

    ("eem6", {
        "d0_cc_n": "AA23",
        "d0_cc_p": "Y23",
        "d1_n": "AB22",
        "d1_p": "AA22",
        "d2_n": "AF32",
        "d2_p": "AE32",
        "d3_n": "AC23",
        "d3_p": "AC22",
        "d4_n": "AE30",
        "d4_p": "AD29",
        "d5_n": "AG32",
        "d5_p": "AG31",
        "d6_n": "AB32",
        "d6_p": "AA32",
        "d7_n": "AB26",
        "d7_p": "AB25",
    }),

    ("eem7", {
        "d0_cc_n": "AC32",
        "d0_cc_p": "AC31",
        "d1_n": "AD34",
        "d1_p": "AC34",
        "d2_n": "AG29",
        "d2_p": "AF29",
        "d3_n": "AB27",
        "d3_p": "AA27",
        "d4_n": "AC27",
        "d4_p": "AC26",
        "d5_n": "AF27",
        "d5_p": "AE27",
        "d6_n": "AG34",
        "d6_p": "AF33",
        "d7_n": "AF28",
        "d7_p": "AE28",
    }),
]


class Platform(XilinxPlatform):
    default_clk_name = "clk50"
    default_clk_period = 20.0

    def __init__(self):
        XilinxPlatform.__init__(self, "xcku040-ffva1156-1-c", _io, _connectors, toolchain="vivado")
        self.toolchain.bitstream_commands.extend([
            "set_property BITSTREAM.CONFIG.OVERTEMPSHUTDOWN Enable [current_design]",
            "set_property BITSTREAM.GENERAL.COMPRESS True [current_design]",
            "set_property BITSTREAM.CONFIG.CONFIGRATE 33 [current_design]",
            "set_property BITSTREAM.CONFIG.SPI_BUSWIDTH 4 [current_design]",
            "set_property CFGBVS VCCO [current_design]",
            "set_property CONFIG_VOLTAGE 3.3 [current_design]",
            ])



from migen.fhdl.structure import *
from migen.fhdl.specials import Memory, _MemoryPort, WRITE_FIRST, NO_CHANGE
from migen.fhdl.decorators import ModuleTransformer
from migen.util.misc import gcd_multiple
from migen.fhdl.bitcontainer import log2_int


class FullMemoryWE(ModuleTransformer):
    def __init__(self):
        self.replacements = dict()

    def transform_fragment(self, i, f):
        newspecials = set()

        for orig in f.specials:
            if not isinstance(orig, Memory):
                newspecials.add(orig)
                continue
            global_granularity = gcd_multiple([p.we_granularity if p.we_granularity else orig.width for p in orig.ports])
            if global_granularity == orig.width:
                newspecials.add(orig)  # nothing to do
            else:
                newmems = []
                for i in range(orig.width//global_granularity):
                    if orig.init is None:
                        newinit = None
                    else:
                        newinit = [(v >> i*global_granularity) & (2**global_granularity - 1) for v in orig.init]
                    newmem = Memory(global_granularity, orig.depth, newinit, orig.name_override + "_grain" + str(i))
                    newspecials.add(newmem)
                    newmems.append(newmem)
                    for port in orig.ports:
                        port_granularity = port.we_granularity if port.we_granularity else orig.width
                        newport = _MemoryPort(
                            adr=port.adr,

                            dat_r=port.dat_r[i*global_granularity:(i+1)*global_granularity] if port.dat_r is not None else None,
                            we=port.we[i*global_granularity//port_granularity] if port.we is not None else None,
                            dat_w=port.dat_w[i*global_granularity:(i+1)*global_granularity] if port.dat_w is not None else None,

                            async_read=port.async_read,
                            re=port.re,
                            we_granularity=0,
                            mode=port.mode,
                            clock_domain=port.clock.cd)
                        newmem.ports.append(newport)
                        newspecials.add(newport)
                self.replacements[orig] = newmems

        f.specials = newspecials
        for oldmem in self.replacements.keys():
            f.specials -= set(oldmem.ports)


class MemoryToArray(ModuleTransformer):
    def __init__(self):
        self.replacements = dict()

    def transform_fragment(self, i, f):
        newspecials = set()
        processed_ports = set()

        for mem in f.specials:
            if not isinstance(mem, Memory):
                newspecials.add(mem)
                continue

            storage = Array()
            self.replacements[mem] = storage
            init = []
            if mem.init is not None:
                init = mem.init
            storage_name = lambda: "%s_data_%d" % (mem.name_override, len(storage))
            for d in init:
                mem_storage = Signal(mem.width, reset=d, name_override=storage_name())
                storage.append(mem_storage)
            for _ in range(mem.depth-len(init)):
                mem_storage = Signal(mem.width, name_override=storage_name())
                storage.append(mem_storage)

            for port in mem.ports:
                try:
                    sync = f.sync[port.clock.cd]
                except KeyError:
                    sync = f.sync[port.clock.cd] = []

                # read
                if port.async_read:
                    f.comb.append(port.dat_r.eq(storage[port.adr]))
                else:
                    if port.mode == WRITE_FIRST:
                        adr_reg = Signal.like(port.adr)
                        rd_stmt = adr_reg.eq(port.adr)
                        f.comb.append(port.dat_r.eq(storage[adr_reg]))
                    elif port.mode == NO_CHANGE and port.we is not None:
                        rd_stmt = If(~port.we, port.dat_r.eq(storage[port.adr]))
                    else: # NO_CHANGE without write capability reduces to READ_FIRST
                        rd_stmt = port.dat_r.eq(storage[port.adr])
                    if port.re is None:
                        sync.append(rd_stmt)
                    else:
                        sync.append(If(port.re, rd_stmt))

                # write
                if port.we is not None:
                    if port.we_granularity:
                        n = mem.width//port.we_granularity
                        for i in range(n):
                            m = i*port.we_granularity
                            M = (i+1)*port.we_granularity
                            sync.append(If(port.we[i],
                                        storage[port.adr][m:M].eq(port.dat_w[m:M])))
                    else:
                        sync.append(If(port.we,
                                       storage[port.adr].eq(port.dat_w)))

                processed_ports.add(port)

        newspecials -= processed_ports
        f.specials = newspecials


class SplitMemory(ModuleTransformer):
    """Split memories with depths that are not powers of two into smaller
    power-of-two memories.

    This prevents toolchains from rounding up and wasting resources."""

    def transform_fragment(self, i, f):
        old_specials, f.specials = f.specials, set()
        old_ports = set()

        for old in old_specials:
            if not isinstance(old, Memory):
                f.specials.add(old)
                continue
            try:
                log2_int(old.depth, need_pow2=True)
                f.specials.add(old)
            except ValueError:
                new, comb, sync = self._split_mem(old)
                old_ports |= set(old.ports)
                f.specials.update(new)
                f.comb += comb
                for cd, sy in sync.items():
                    s = f.sync.setdefault(cd, [])
                    s += sy
        f.specials -= old_ports

    def _split_mem(self, mem):
        depths = [1 << i for i in range(log2_int(mem.depth, need_pow2=False))
                  if mem.depth & (1 << i)]
        depths.reverse()
        inits = None
        if mem.init is not None:
            inits = list(mem.init)
        mems = []
        for i, depth in enumerate(depths):
            init = None
            if inits is not None:
                init = inits[:depth]
                del inits[:depth]
            name = "{}_part{}".format(mem.name_override, i)
            mems.append(Memory(width=mem.width, depth=depth,
                               init=init, name=name))
        ports = []
        comb = []
        sync = {}
        for port in mem.ports:
            p, c, s = self._split_port(port, mems)
            ports += p
            comb += c
            sy = sync.setdefault(port.clock.cd, [])
            sy += s
        return mems + ports, comb, sync

    def _split_port(self, port, mems):
        ports = [mem.get_port(write_capable=port.we is not None,
                              async_read=port.async_read,
                              has_re=port.re is not None,
                              we_granularity=port.we_granularity,
                              mode=port.mode,
                              clock_domain=port.clock.cd)
                 for mem in mems]

        sel = Signal(max=len(ports), reset=len(ports) - 1)
        sel_r = Signal.like(sel)
        eq = sel_r.eq(sel)
        if port.re is not None:
            eq = If(port.re, eq)
        comb, sync = [], []
        if port.async_read:
            comb += [eq]
        else:
            sync += [eq]
        comb += reversed([If(~port.adr[len(p.adr)], sel.eq(i))
                          for i, p in enumerate(ports)])
        comb += [p.adr.eq(port.adr) for p in ports]
        comb.append(port.dat_r.eq(Array([p.dat_r for p in ports])[sel_r]))
        if port.we is not None:
            comb.append(Array([p.we for p in ports])[sel].eq(port.we))
            comb += [p.dat_w.eq(port.dat_w) for p in ports]
        if port.re is not None:
            comb += [p.re.eq(port.re) for p in ports]
        return ports, comb, sync



""" Platform definitions for apf6sp board by Armadeus System """
from migen import *
from migen.fhdl import *
from migen.build.generic_platform import *
from migen.build.altera import AlteraPlatform

_io = [("pcie_x1", 0,
        Subsignal("clk_p", Pins("V4"), IOStandard("1.5-V PCML")),
        Subsignal("clk_n", Pins("U4"), IOStandard("1.5-V PCML")),
        Subsignal("rx_p", Pins("AA2"), IOStandard("1.5-V PCML")),
        Subsignal("rx_n", Pins("AA1"), IOStandard("1.5-V PCML")),
        Subsignal("tx_p", Pins("Y4"), IOStandard("1.5-V PCML")),
        Subsignal("tx_n", Pins("Y3"), IOStandard("1.5-V PCML")),
        Subsignal("perst", Pins("R17"), IOStandard("2.5 V")),
        Subsignal("npor", Pins("R16"), IOStandard("2.5 V"))),

       ("pcie_clk", 0,
        Subsignal("p", Pins("V4"), IOStandard("1.5-V PCML")),
        Subsignal("n", Pins("U4"), IOStandard("1.5-V PCML"))),

       ("ddram", 0,
        Subsignal("a", Pins("C11 B11 A8 A7 " +
                            "D11 E11 F8 E7 " +
                            "D9 D8 B6 B5 C8 " +
                            "B8 H6"), IOStandard("SSTL-135")),

        Subsignal("ba", Pins("C6 C10 C9"), IOStandard("SSTL-135")),

        Subsignal("ras_n", Pins("A9"), IOStandard("SSTL-135")),
        Subsignal("cas_n", Pins("A10"), IOStandard("SSTL-135")),
        Subsignal("cs_n", Pins("H8"), IOStandard("SSTL-135")),
        Subsignal("we_n", Pins("E6"), IOStandard("SSTL-135")),

        Subsignal("dm", Pins("A15 C19 C21 F12 " +
                             "E12 B12 B13 C13 " +
                             "D13 C14 A14 E14 " +
                             "F15 B18 A17 C15 " +
                             "C16 B16 C18 D17 " +
                             "E16 A20 A22 E17 " +
                             "D18 B20 C20"), IOStandard("SSTL-135")),

        Subsignal("dqs_n",
                  Pins("G8 H12 G14"),
                  IOStandard("SSTL-135")),

        Subsignal("dqs",
                  Pins("H9 G12 G15"),
                  IOStandard("SSTL-135")),

        Subsignal("ck", Pins("J9"), IOStandard("SSTL-135")),
        Subsignal("ck_n", Pins("J8"), IOStandard("SSTL-135")),
        Subsignal("cke", Pins("B15"), IOStandard("SSTL-135")),
        Subsignal("odt", Pins("A13"), IOStandard("SSTL-135")),

        Subsignal("reset_n", Pins("B22"), IOStandard("SSTL-135")),
        Subsignal("oct_rzqin", Pins("A12"), IOStandard("SSTL-135")))]

_connector = [
    ("HIROSE",
     {"CLKIN0": "T10",
      "CLKIN_1N": "R9",
      "CLKIN_1P": "P9",
      "CLKIN_2N": "R15",
      "CLKIN_2P": "T15",
      "CLKOUT0": "R10",
      "CLKOUT_1N": "G17",
      "CLKOUT_1P": "G18",
      "CLKOUT_2N": "AB22",
      "CLKOUT_2P": "AA22",
      "D0": "M10",
      "D1": "K15",
      "D2": "L9",
      "D3": "L15",
      "LVDS_RX_N0": "AA9",
      "LVDS_RX_P0": "Y9",
      "LVDS_RX_N1": "U10",
      "LVDS_RX_P1": "U11",
      "LVDS_RX_N2": "U8",
      "LVDS_RX_P2": "V9",
      "LVDS_RX_N3": "U12",
      "LVDS_RX_P3": "T12",
      "LVDS_RX_N4": "W13",
      "LVDS_RX_P4": "V13",
      "LVDS_RX_N5": "W12",
      "LVDS_RX_P5": "Y12",
      "LVDS_RX_N6": "Y14",
      "LVDS_RX_P6": "W14",
      "LVDS_RX_N7": "U16",
      "LVDS_RX_P7": "U17",
      "LVDS_RX_N8": "Y16",
      "LVDS_RX_P8": "Y17",
      "LVDS_RX_N9": "W16",
      "LVDS_RX_P9": "W17",
      "LVDS_RX_N10": "V18",
      "LVDS_RX_P10": "W18",
      "LVDS_RX_N11": "V19",
      "LVDS_RX_P11": "V20",
      "LVDS_RX_N12": "T18",
      "LVDS_RX_P12": "T17",
      "LVDS_RX_N13": "J18",
      "LVDS_RX_P13": "J17",
      "LVDS_RX_N14": "K19",
      "LVDS_RX_P14": "L18",
      "LVDS_RX_N15": "L20",
      "LVDS_RX_P15": "L19",
      "LVDS_RX_N16": "P19",
      "LVDS_RX_P16": "R19",
      "LVDS_TX_N0": "W11",
      "LVDS_TX_P0": "Y11",
      "LVDS_TX_N1": "R12",
      "LVDS_TX_P1": "T13",
      "LVDS_TX_N2": "AA13",
      "LVDS_TX_P2": "AB13",
      "LVDS_TX_N3": "AB10",
      "LVDS_TX_P3": "AB11",
      "LVDS_TX_N4": "Y10",
      "LVDS_TX_P4": "AA10",
      "LVDS_TX_N5": "AA15",
      "LVDS_TX_P5": "Y15",
      "LVDS_TX_N6": "AB15",
      "LVDS_TX_P6": "AB16",
      "LVDS_TX_N7": "R11",
      "LVDS_TX_P7": "P12",
      "LVDS_TX_N8": "AA17",
      "LVDS_TX_P8": "AB17",
      "LVDS_TX_N9": "AB18",
      "LVDS_TX_P9": "AA18",
      "LVDS_TX_N10": "AA19",
      "LVDS_TX_P10": "Y19",
      "LVDS_TX_N11": "R21",
      "LVDS_TX_P11": "R20",
      "LVDS_TX_N12": "AB20",
      "LVDS_TX_P12": "AB21",
      "LVDS_TX_N13": "AA20",
      "LVDS_TX_P13": "Y20",
      "LVDS_TX_N14": "U21",
      "LVDS_TX_P14": "U22",
      "LVDS_TX_N15": "R22",
      "LVDS_TX_P15": "T22",
      "LVDS_TX_N16": "G21",
      "LVDS_TX_P16": "G22",
      "XCVR_RX_N0": "W1",
      "XCVR_RX_P0": "W2",
      "XCVR_RX_N1": "R1",
      "XCVR_RX_P1": "R2",
      "XCVR_RX_N2": "L1",
      "XCVR_RX_P2": "L2",
      "XCVR_TX_N0": "U1",
      "XCVR_TX_P0": "U2",
      "XCVR_TX_N1": "N1",
      "XCVR_TX_P1": "N2",
      "XCVR_TX_N2": "J1",
      "XCVR_TX_P2": "J2",
     }
    )]

class Platform(AlteraPlatform):
    """ Apf6sp platform """
    default_clk_name = "pcie_clk"
    default_clk_period = 8

    def __init__(self, toolchain="quartus"):
        AlteraPlatform.__init__(self, "5CGXFC4C7U19C8", _io, _connector)


class PciePllClockedModule(Module):
    """ This function add the pcie PLL as main clock for modules.
    See http://www.armadeus.com/wiki/index.php?title=IMX6-CycloneV_interface_description#Clocking_without_PCIe
    for the explainations
    """

    def __init__(self, platform):
        cd_sys = ClockDomain(name="cd_sys", reset_less=True)
        self.clock_domains += cd_sys

        pcie_clk = platform.request("pcie_clk")
        pci_clk_sig = Signal()

        self.specials += Instance("ALT_INBUF_DIFF", name="ibuf_diff",
                                  i_i=pcie_clk.p,
                                  i_ibar=pcie_clk.n,
                                  o_o=pci_clk_sig)
        self.specials += Instance("altera_pll",
                        i_rst=0,
                        o_outclk=cd_sys.clk,
                        i_refclk=pci_clk_sig,

                        p_fractional_vco_multiplier="false",
                        p_reference_clock_frequency="125.0 MHz",
                        p_operation_mode="direct",
                        p_number_of_clocks=Instance.PreformattedParam("1"),
                        p_output_clock_frequency0="62.500000 MHz",
                        p_phase_shift0="0 ps",
                        p_duty_cycle0=Instance.PreformattedParam("50"),
                        p_output_clock_frequency1="0 MHz",
                        p_phase_shift1="0 ps",
                        p_duty_cycle1=Instance.PreformattedParam("50"),
                        p_output_clock_frequency2="0 MHz",
                        p_phase_shift2="0 ps",
                        p_duty_cycle2=Instance.PreformattedParam("50"),
                        p_output_clock_frequency3="0 MHz",
                        p_phase_shift3="0 ps",
                        p_duty_cycle3=Instance.PreformattedParam("50"),
                        p_output_clock_frequency4="0 MHz",
                        p_phase_shift4="0 ps",
                        p_duty_cycle4=Instance.PreformattedParam("50"),
                        p_output_clock_frequency5="0 MHz",
                        p_phase_shift5="0 ps",
                        p_duty_cycle5=Instance.PreformattedParam("50"),
                        p_output_clock_frequency6="0 MHz",
                        p_phase_shift6="0 ps",
                        p_duty_cycle6=Instance.PreformattedParam("50"),
                        p_output_clock_frequency7="0 MHz",
                        p_phase_shift7="0 ps",
                        p_duty_cycle7=Instance.PreformattedParam("50"),
                        p_output_clock_frequency8="0 MHz",
                        p_phase_shift8="0 ps",
                        p_duty_cycle8=Instance.PreformattedParam("50"),
                        p_output_clock_frequency9="0 MHz",
                        p_phase_shift9="0 ps",
                        p_duty_cycle9=Instance.PreformattedParam("50"),
                        p_output_clock_frequency10="0 MHz",
                        p_phase_shift10="0 ps",
                        p_duty_cycle10=Instance.PreformattedParam("50"),
                        p_output_clock_frequency11="0 MHz",
                        p_phase_shift11="0 ps",
                        p_duty_cycle11=Instance.PreformattedParam("50"),
                        p_output_clock_frequency12="0 MHz",
                        p_phase_shift12="0 ps",
                        p_duty_cycle12=Instance.PreformattedParam("50"),
                        p_output_clock_frequency13="0 MHz",
                        p_phase_shift13="0 ps",
                        p_duty_cycle13=Instance.PreformattedParam("50"),
                        p_output_clock_frequency14="0 MHz",
                        p_phase_shift14="0 ps",
                        p_duty_cycle14=Instance.PreformattedParam("50"),
                        p_output_clock_frequency15="0 MHz",
                        p_phase_shift15="0 ps",
                        p_duty_cycle15=Instance.PreformattedParam("50"),
                        p_output_clock_frequency16="0 MHz",
                        p_phase_shift16="0 ps",
                        p_duty_cycle16=Instance.PreformattedParam("50"),
                        p_output_clock_frequency17="0 MHz",
                        p_phase_shift17="0 ps",
                        p_pll_subtype="General",
                        p_pll_type="General",
                        p_duty_cycle17=Instance.PreformattedParam("50"))



import cairo
import math


def _cairo_draw_node(ctx, dx, radius, color, outer_color, s):
    ctx.save()

    ctx.translate(dx, 0)

    ctx.set_line_width(0.0)
    gradient_color = cairo.RadialGradient(0, 0, 0, 0, 0, radius)
    gradient_color.add_color_stop_rgb(0, *color)
    gradient_color.add_color_stop_rgb(1, *outer_color)
    ctx.set_source(gradient_color)
    ctx.arc(0, 0, radius, 0, 2*math.pi)
    ctx.fill()

    lines = s.split("\n")
    textws = []
    texths = []
    for line in lines:
        x_bearing, y_bearing, w, h, x_advance, y_advance = ctx.text_extents(line)
        textws.append(w)
        texths.append(h + 2)
    ctx.translate(0, -sum(texths[1:])/2)
    for line, w, h in zip(lines, textws, texths):
        ctx.translate(-w/2, h/2)
        ctx.move_to(0, 0)
        ctx.set_source_rgb(0, 0, 0)
        ctx.show_text(line)
        ctx.translate(w/2, h/2)

    ctx.restore()


def _cairo_draw_connection(ctx, x0, y0, color0, x1, y1, color1):
    ctx.move_to(x0, y0)
    ctx.curve_to(x0, y0+20, x1, y1-20, x1, y1)
    ctx.set_line_width(1.2)
    gradient_color = cairo.LinearGradient(x0, y0, x1, y1)
    gradient_color.add_color_stop_rgb(0, *color0)
    gradient_color.add_color_stop_rgb(1, *color1)
    ctx.set_source(gradient_color)
    ctx.stroke()


class RenderNode:
    def __init__(self, label, children=None, color=(0.8, 0.8, 0.8), radius=40):
        self.label = label
        if children is None:
            children = []
        self.children = children
        self.color = color
        self.outer_color = (color[0]*3/5, color[1]*3/5, color[2]*3/5)
        self.radius = radius
        self.pitch = self.radius*3

    def get_dimensions(self):
        if self.children:
            cws, chs, cdxs = zip(*[c.get_dimensions() for c in self.children])
            w = sum(cws)
            h = self.pitch + max(chs)
            dx = cws[0]/4 - cws[-1]/4
        else:
            w = h = self.pitch
            dx = 0
        return w, h, dx

    def render(self, ctx):
        if self.children:
            cws, chs, cdxs = zip(*[c.get_dimensions() for c in self.children])
            first_child_x = -sum(cws)/2

            ctx.save()
            ctx.translate(first_child_x, self.pitch)
            for c, w in zip(self.children, cws):
                ctx.translate(w/2, 0)
                c.render(ctx)
                ctx.translate(w/2, 0)
            ctx.restore()

            dx = cws[0]/4 - cws[-1]/4

            current_x = first_child_x
            for c, w, cdx in zip(self.children, cws, cdxs):
                current_y = self.pitch - c.radius
                current_x += w/2
                _cairo_draw_connection(ctx, dx, self.radius, self.outer_color, current_x+cdx, current_y, c.outer_color)
                current_x += w/2
        else:
            dx = 0
        _cairo_draw_node(ctx, dx, self.radius, self.color, self.outer_color, self.label)

    def to_svg(self, name):
        w, h, dx = self.get_dimensions()
        surface = cairo.SVGSurface(name, w, h)
        ctx = cairo.Context(surface)
        ctx.translate(w/2, self.pitch/2)
        self.render(ctx)
        surface.finish()


def _test():
    xns = [RenderNode("X"+str(n)) for n in range(5)]
    yns = [RenderNode("Y"+str(n), [RenderNode("foo", color=(0.1*n, 0.5+0.2*n, 1.0-0.3*n))]) for n in range(3)]
    n1 = RenderNode("n1", yns)
    n2 = RenderNode("n2", xns, color=(0.8, 0.5, 0.9))
    top = RenderNode("top", [n1, n2])
    top.to_svg("test.svg")

if __name__ == "__main__":
    _test()



from migen.build.generic_platform import *
from migen.build.xilinx import XilinxPlatform


_io = [
    ("user_led", 1, Pins("T9"), IOStandard("LVCMOS33"), Drive(24), Misc("SLEW=QUIETIO")),
    ("user_led", 3, Pins("R9"), IOStandard("LVCMOS33"), Drive(24), Misc("SLEW=QUIETIO")),

    ("clk50", 0, Pins("A10"), IOStandard("LVCMOS33")),

    ("serial", 0,
        Subsignal("tx", Pins("P12"), IOStandard("LVCMOS33"), Misc("SLEW=SLOW")),
        Subsignal("rx", Pins("M11"), IOStandard("LVCMOS33"), Misc("PULLUP"))
    ),

    ("spiflash", 0,
        Subsignal("cs_n", Pins("T3")),
        Subsignal("clk", Pins("R11")),
        Subsignal("mosi", Pins("T10")),
        Subsignal("miso", Pins("P10"), Misc("PULLUP")),
        IOStandard("LVCMOS33"), Misc("SLEW=FAST")
    ),

    ("sdram_clock", 0, Pins("H1"), IOStandard("LVCMOS33"), Misc("SLEW=FAST")),
    ("sdram", 0,
        Subsignal("a", Pins("L4 M3 M4 N3 R2 R1 P2 P1 N1 M1 L3 L1 K1")),
        Subsignal("ba", Pins("K3 K2")),
        Subsignal("cs_n", Pins("J3")),
        Subsignal("cke", Pins("J1")),
        Subsignal("ras_n", Pins("J4")),
        Subsignal("cas_n", Pins("H3")),
        Subsignal("we_n", Pins("G3")),
        Subsignal("dq", Pins("A3 A2 B3 B2 C3 C2 D3 E3 G1 F1 F2 E1 E2 D1 C1 B1")),
        Subsignal("dm", Pins("F3 H2")),
        IOStandard("LVCMOS33"), Misc("SLEW=FAST")
    )
]

_connectors = [
    ("U7", "- - - - - - - E12 E13 B15 B16 C15 C16 D14 D16 E15 E16 F15 F16 G11 F12 " +
    "F14 F13 G16 G14 H15 H16 G12 H11 H13 H14 J14 J16 J11 J12 K14 J13 K15 K16 " +
    "L16 L14 K11 K12 M15 M16 N14 N16 M13 M14 L12 L13 P15 P16 R15 R16 R14 T15 " +
    "T13 T14 T12 R12"),
    ("U8", "- - - - - - - A14 B14 C13 A13 B12 A12 C11 A11 B10 A9 C9 A8 B8 A7 C7 A6 " +
     "B6 A5 B5 A4 E10 C10 E11 F10 F9 D9 C8 D8 E7 E6 F7 C6 D6 M6 P4 N5 P5 N6 M7 " +
     "P6 N8 L7 P9 T4 T5 R5 T6 T7 N9 M9 M10 P11 P12 M11"), #P12 M11 used as serial
]


class Platform(XilinxPlatform):
    default_clk_name = "clk50"
    default_clk_period = 50

    def __init__(self):
        XilinxPlatform.__init__(self, "xc6slx16-ftg256", _io, _connectors)




from migen.build.generic_platform import *
from migen.build.xilinx import XilinxPlatform
from migen.build.xilinx.programmer import UrJTAG


_io = [
    ("user_led", 0, Pins("B16"), IOStandard("LVCMOS33"), Drive(24), Misc("SLEW=QUIETIO")),
    ("user_led", 1, Pins("A16"), IOStandard("LVCMOS33"), Drive(24), Misc("SLEW=QUIETIO")),

    ("user_btn", 0, Pins("AB4"), IOStandard("LVCMOS33")),
    ("user_btn", 1, Pins("AA4"), IOStandard("LVCMOS33")),
    ("user_btn", 2, Pins("AB5"), IOStandard("LVCMOS33")),

    ("clk50", 0, Pins("AB11"), IOStandard("LVCMOS33")),

    # When executing softcore code in-place from the flash, we want
    # the flash reset to be released before the system reset.
    ("norflash_rst_n", 0, Pins("P22"), IOStandard("LVCMOS33"), Misc("SLEW=FAST"), Drive(8)),
    ("norflash", 0,
        Subsignal("adr", Pins("L22 L20 K22 K21 J19 H20 F22",
            "F21 K17 J17 E22 E20 H18 H19 F20",
            "G19 C22 C20 D22 D21 F19 F18 D20 D19")),
        Subsignal("d", Pins("AA20 U14 U13 AA6 AB6 W4 Y4 Y7",
            "AA2 AB2 V15 AA18 AB18 Y13 AA12 AB12"), Misc("PULLDOWN")),
        Subsignal("oe_n", Pins("M22")),
        Subsignal("we_n", Pins("N20")),
        Subsignal("ce_n", Pins("M21")),
        IOStandard("LVCMOS33"), Misc("SLEW=FAST"), Drive(8)
    ),

    ("serial", 0,
        Subsignal("tx", Pins("L17"), IOStandard("LVCMOS33"), Misc("SLEW=SLOW")),
        Subsignal("rx", Pins("K18"), IOStandard("LVCMOS33"), Misc("PULLUP"))
    ),

    ("ddram_clock", 0,
        Subsignal("p", Pins("M3")),
        Subsignal("n", Pins("L4")),
        IOStandard("SSTL2_I")
    ),
    ("ddram", 0,
        Subsignal("a", Pins("B1 B2 H8 J7 E4 D5 K7 F5 G6 C1 C3 D1 D2")),
        Subsignal("ba", Pins("A2 E6")),
        Subsignal("cs_n", Pins("F7")),
        Subsignal("cke", Pins("G7")),
        Subsignal("ras_n", Pins("E5")),
        Subsignal("cas_n", Pins("C4")),
        Subsignal("we_n", Pins("D3")),
        Subsignal("dq", Pins("Y2 W3 W1 P8 P7 P6 P5 T4 T3",
            "U4 V3 N6 N7 M7 M8 R4 P4 M6 L6 P3 N4",
            "M5 V2 V1 U3 U1 T2 T1 R3 R1 P2 P1")),
        Subsignal("dm", Pins("E1 E3 F3 G4")),
        Subsignal("dqs", Pins("F1 F2 H5 H6")),
        IOStandard("SSTL2_I")
    ),

    ("eth_clocks", 0,
        Subsignal("phy", Pins("M20")),
        Subsignal("rx", Pins("H22")),
        Subsignal("tx", Pins("H21")),
        IOStandard("LVCMOS33")
    ),
    ("eth", 0,
        Subsignal("rst_n", Pins("R22")),
        Subsignal("rx_dv", Pins("V21")),
        Subsignal("rx_er", Pins("V22")),
        Subsignal("rx_data", Pins("U22 U20 T22 T21")),
        Subsignal("tx_en", Pins("N19")),
        Subsignal("tx_er", Pins("M19")),
        Subsignal("tx_data", Pins("M16 L15 P19 P20")),
        Subsignal("col", Pins("W20")),
        Subsignal("crs", Pins("W22")),
        IOStandard("LVCMOS33")
    ),

    ("vga_out", 0,
        Subsignal("clk", Pins("A11")),
        Subsignal("r", Pins("C6 B6 A6 C7 A7 B8 A8 D9")),
        Subsignal("g", Pins("C8 C9 A9 D7 D8 D10 C10 B10")),
        Subsignal("b", Pins("D11 C12 B12 A12 C13 A13 D14 C14")),
        Subsignal("hsync_n", Pins("A14")),
        Subsignal("vsync_n", Pins("C15")),
        Subsignal("psave_n", Pins("B14")),
        IOStandard("LVCMOS33")
    ),

    ("mmc", 0,
        Subsignal("clk", Pins("A10")),
        Subsignal("cmd", Pins("B18")),
        Subsignal("dat", Pins("A18 E16 C17 A17")),
        IOStandard("LVCMOS33")
    ),

    # Digital video mixer extension board
    ("dvi_in", 0,
        Subsignal("clk", Pins("A20")),
        Subsignal("data0_n", Pins("A21")),
        Subsignal("data1", Pins("B21")),
        Subsignal("data2_n", Pins("B22")),
        Subsignal("scl", Pins("G16")),
        Subsignal("sda", Pins("G17")),
        IOStandard("LVCMOS33")
    ),
    ("dvi_in", 1,
        Subsignal("clk", Pins("H17")),
        Subsignal("data0_n", Pins("H16")),
        Subsignal("data1", Pins("F17")),
        Subsignal("data2_n", Pins("F16")),
        Subsignal("scl", Pins("J16")),
        Subsignal("sda", Pins("K16")),
        IOStandard("LVCMOS33")
    ),
    ("dvi_pots", 0,
        Subsignal("charge", Pins("A18")),        # SD_DAT0
        Subsignal("blackout", Pins("C17")),        # SD_DAT2
        Subsignal("crossfade", Pins("A17")),    # SD_DAT3
        IOStandard("LVCMOS33")
    )
]


class Platform(XilinxPlatform):
    default_clk_name = "clk50"
    default_clk_period = 20

    def __init__(self):
        XilinxPlatform.__init__(self, "xc6slx45-fgg484-2", _io)

    def create_programmer(self):
        return UrJTAG(cable="milkymist", flash_proxy_basename="fjmem-m1.bit")

    def do_finalize(self, fragment):
        XilinxPlatform.do_finalize(self, fragment)

        try:
            eth_clocks = self.lookup_request("eth_clocks")
            self.add_period_constraint(eth_clocks.rx, 40)
            self.add_period_constraint(eth_clocks.tx, 40)
            self.add_platform_command("""
TIMESPEC "TS{phy_tx_clk}_io" = FROM "GRP{phy_tx_clk}" TO "PADS" 10 ns;
TIMESPEC "TS{phy_rx_clk}_io" = FROM "PADS" TO "GRP{phy_rx_clk}" 10 ns;
""", phy_rx_clk=eth_clocks.rx, phy_tx_clk=eth_clocks.tx)
        except ConstraintError:
            pass

        for i in range(2):
            si = "dviclk"+str(i)
            try:
                self.add_period_constraint(self.lookup_request("dvi_in", i).clk, 26.7)
            except ConstraintError:
                pass



from migen.fhdl.structure import *
from migen.fhdl.specials import Special


__all__ = ["AsyncResetSynchronizer"]


class AsyncResetSynchronizer(Special):
    def __init__(self, cd, async_reset):
        Special.__init__(self)
        self.cd = cd
        self.async_reset = wrap(async_reset)

    def iter_expressions(self):
        yield self.cd, "clk", SPECIAL_INPUT
        yield self.cd, "rst", SPECIAL_OUTPUT
        yield self, "async_reset", SPECIAL_INPUT

    @staticmethod
    def lower(dr):
        raise NotImplementedError("Attempted to use a reset synchronizer, but platform does not support them")



from migen.fhdl import structure as f


__all__ = ["log2_int", "bits_for", "value_bits_sign"]


def log2_int(n, need_pow2=True):
    if n == 0:
        return 0
    r = (n - 1).bit_length()
    if need_pow2 and (1 << r) != n:
        raise ValueError("Not a power of 2")
    return r


def bits_for(n, require_sign_bit=False):
    if n > 0:
        r = log2_int(n + 1, False)
    else:
        require_sign_bit = True
        r = log2_int(-n, False)
    if require_sign_bit:
        r += 1
    return r


def _bitwise_binary_bits_sign(a, b):
    if not a[1] and not b[1]:
        # both operands unsigned
        return max(a[0], b[0]), False
    elif a[1] and b[1]:
        # both operands signed
        return max(a[0], b[0]), True
    elif not a[1] and b[1]:
        # first operand unsigned (add sign bit), second operand signed
        return max(a[0] + 1, b[0]), True
    else:
        # first signed, second operand unsigned (add sign bit)
        return max(a[0], b[0] + 1), True


def value_bits_sign(v):
    """Bit length and signedness of a value.

    Parameters
    ----------
    v : Value

    Returns
    -------
    int, bool
        Number of bits required to store `v` or available in `v`, followed by
        whether `v` has a sign bit (included in the bit count).

    Examples
    --------
    >>> value_bits_sign(f.Signal(8))
    8, False
    >>> value_bits_sign(C(0xaa))
    8, False
    """
    if isinstance(v, (f.Constant, f.Signal)):
        return v.nbits, v.signed
    elif isinstance(v, (f.ClockSignal, f.ResetSignal)):
        return 1, False
    elif isinstance(v, f._Operator):
        obs = list(map(value_bits_sign, v.operands))
        if v.op == "+" or v.op == "-":
            if len(obs) == 1:
                if v.op == "-" and not obs[0][1]:
                    return obs[0][0] + 1, True
                else:
                    return obs[0]
            n, s = _bitwise_binary_bits_sign(*obs)
            return n + 1, s
        elif v.op == "*":
            if not obs[0][1] and not obs[1][1]:
                # both operands unsigned
                return obs[0][0] + obs[1][0], False
            elif obs[0][1] and obs[1][1]:
                # both operands signed
                return obs[0][0] + obs[1][0] - 1, True
            else:
                # one operand signed, the other unsigned (add sign bit)
                return obs[0][0] + obs[1][0] + 1 - 1, True
        elif v.op == "<<<":
            if obs[1][1]:
                extra = 2**(obs[1][0] - 1) - 1
            else:
                extra = 2**obs[1][0] - 1
            return obs[0][0] + extra, obs[0][1]
        elif v.op == ">>>":
            if obs[1][1]:
                extra = 2**(obs[1][0] - 1)
            else:
                extra = 0
            return obs[0][0] + extra, obs[0][1]
        elif v.op == "&" or v.op == "^" or v.op == "|":
            return _bitwise_binary_bits_sign(*obs)
        elif (v.op == "<" or v.op == "<=" or v.op == "==" or v.op == "!=" or
              v.op == ">" or v.op == ">="):
            return 1, False
        elif v.op == "~":
            return obs[0]
        elif v.op == "m":
            return _bitwise_binary_bits_sign(obs[1], obs[2])
        else:
            raise TypeError
    elif isinstance(v, f._Slice):
        return v.stop - v.start, False
    elif isinstance(v, f._Part):
        return v.width, False
    elif isinstance(v, f.Cat):
        return sum(value_bits_sign(sv)[0] for sv in v.l), False
    elif isinstance(v, f.Replicate):
        return (value_bits_sign(v.v)[0])*v.n, False
    elif isinstance(v, f._ArrayProxy):
        bsc = list(map(value_bits_sign, v.choices))
        return max(bs[0] for bs in bsc), any(bs[1] for bs in bsc)
    else:
        raise TypeError("Can not calculate bit length of {} {}".format(
            type(v), v))



import builtins as _builtins
import collections.abc as _collections_abc
import collections as _collections
import re as _re

from migen.fhdl import tracer as _tracer
from migen.util.misc import flat_iteration as _flat_iteration


class DUID:
    """Deterministic Unique IDentifier"""
    __next_uid = 0
    def __init__(self):
        self.duid = DUID.__next_uid
        DUID.__next_uid += 1


class _Value(DUID):
    """Base class for operands

    Instances of `_Value` or its subclasses can be operands to
    arithmetic, comparison, bitwise, and logic operators.
    They can be assigned (:meth:`eq`) or indexed/sliced (using the usual
    Python indexing and slicing notation).

    Values created from integers have the minimum bit width to necessary to
    represent the integer.
    """
    def __bool__(self):
        # Special case: Constants and Signals are part of a set or used as
        # dictionary keys, and Python needs to check for equality.
        if isinstance(self, _Operator) and self.op == "==":
            a, b = self.operands
            if isinstance(a, Constant) and isinstance(b, Constant):
                return a.value == b.value
            if isinstance(a, Signal) and isinstance(b, Signal):
                return a is b
            if (isinstance(a, Constant) and isinstance(b, Signal)
                    or isinstance(a, Signal) and isinstance(b, Constant)):
                return False
        raise TypeError("Attempted to convert Migen value to boolean")

    def __invert__(self):
        return _Operator("~", [self])
    def __neg__(self):
        return _Operator("-", [self])

    def __add__(self, other):
        return _Operator("+", [self, other])
    def __radd__(self, other):
        return _Operator("+", [other, self])
    def __sub__(self, other):
        return _Operator("-", [self, other])
    def __rsub__(self, other):
        return _Operator("-", [other, self])
    def __mul__(self, other):
        return _Operator("*", [self, other])
    def __rmul__(self, other):
        return _Operator("*", [other, self])
    def __lshift__(self, other):
        return _Operator("<<<", [self, other])
    def __rlshift__(self, other):
        return _Operator("<<<", [other, self])
    def __rshift__(self, other):
        return _Operator(">>>", [self, other])
    def __rrshift__(self, other):
        return _Operator(">>>", [other, self])
    def __and__(self, other):
        return _Operator("&", [self, other])
    def __rand__(self, other):
        return _Operator("&", [other, self])
    def __xor__(self, other):
        return _Operator("^", [self, other])
    def __rxor__(self, other):
        return _Operator("^", [other, self])
    def __or__(self, other):
        return _Operator("|", [self, other])
    def __ror__(self, other):
        return _Operator("|", [other, self])

    def __lt__(self, other):
        return _Operator("<", [self, other])
    def __le__(self, other):
        return _Operator("<=", [self, other])
    def __eq__(self, other):
        return _Operator("==", [self, other])
    def __ne__(self, other):
        return _Operator("!=", [self, other])
    def __gt__(self, other):
        return _Operator(">", [self, other])
    def __ge__(self, other):
        return _Operator(">=", [self, other])

    def __len__(self):
        from migen.fhdl.bitcontainer import value_bits_sign
        return value_bits_sign(self)[0]

    def __getitem__(self, key):
        n = len(self)
        if isinstance(key, int):
            if key >= n:
                raise IndexError
            if key < 0:
                key += n
            return _Slice(self, key, key+1)
        elif isinstance(key, slice):
            start, stop, step = key.indices(n)
            if step != 1:
                return Cat(self[i] for i in range(start, stop, step))
            return _Slice(self, start, stop)
        else:
            raise TypeError("Cannot use type {} ({}) as key".format(
                type(key), repr(key)))

    def eq(self, r):
        """Assignment

        Parameters
        ----------
        r : _Value, in
            Value to be assigned.

        Returns
        -------
        _Assign
            Assignment statement that can be used in combinatorial or
            synchronous context.
        """
        return _Assign(self, r)

    def part(self, offset, width):
        """Indexed part-select
        Selects a constant width but variable offset part of a ``_Value``

        Parameters
        ----------
        offset : _Value, in
            start point of the selected bits
        width : Constant, in
            number of selected bits

        Returns
        -------
        _Part
            Selected part of the ``_Value``
        """
        offset = wrap(offset)
        return _Part(self, offset, width)

    def __hash__(self):
        raise TypeError("unhashable type: '{}'".format(type(self).__name__))


def wrap(value):
    """Ensures that the passed object is a Migen value. Booleans and integers
    are automatically wrapped into ``Constant``."""
    if isinstance(value, (bool, int)):
        value = Constant(value)
    if not isinstance(value, _Value):
        raise TypeError("Object '{}' of type {} is not a Migen value"
                        .format(value, type(value)))
    return value


class _Operator(_Value):
    def __init__(self, op, operands):
        _Value.__init__(self)
        self.op = op
        self.operands = [wrap(o) for o in operands]


def Mux(sel, val1, val0):
    """Multiplex between two values

    Parameters
    ----------
    sel : _Value(1), in
        Selector.
    val1 : _Value(N), in
    val0 : _Value(N), in
        Input values.

    Returns
    -------
    _Value(N), out
        Output `_Value`. If `sel` is asserted, the Mux returns
        `val1`, else `val0`.
    """
    return _Operator("m", [sel, val1, val0])


class _Slice(_Value):
    def __init__(self, value, start, stop):
        _Value.__init__(self)
        if not isinstance(start, int) or not isinstance(stop, int):
            raise TypeError("Slice boundaries must be integers")
        self.value = wrap(value)
        self.start = start
        self.stop = stop


class _Part(_Value):
    def __init__(self, value, offset, width):
        _Value.__init__(self)
        if not isinstance(width, int):
            raise TypeError("Cannot use non int width {} ({}) for part".format(
                width, repr(width)))
        if not isinstance(offset, Constant) and not isinstance(offset, _Value):
            raise TypeError("Must use Value or Constant offset instead of {} ({}) for part".format(
                offset, repr(offset)))
        self.value = value
        self.offset = offset
        self.width = width

class Cat(_Value):
    """Concatenate values

    Form a compound `_Value` from several smaller ones by concatenation.
    The first argument occupies the lower bits of the result.
    The return value can be used on either side of an assignment, that
    is, the concatenated value can be used as an argument on the RHS or
    as a target on the LHS. If it is used on the LHS, it must solely
    consist of `Signal` s, slices of `Signal` s, and other concatenations
    meeting these properties. The bit length of the return value is the sum of
    the bit lengths of the arguments::

        len(Cat(args)) == sum(len(arg) for arg in args)

    Parameters
    ----------
    *args : _Values or iterables of _Values, inout
        `_Value` s to be concatenated.

    Returns
    -------
    Cat, inout
        Resulting `_Value` obtained by concatentation.
    """
    def __init__(self, *args):
        _Value.__init__(self)
        self.l = [wrap(v) for v in _flat_iteration(args)]


class Replicate(_Value):
    """Replicate a value

    An input value is replicated (repeated) several times
    to be used on the RHS of assignments::

        len(Replicate(s, n)) == len(s)*n

    Parameters
    ----------
    v : _Value, in
        Input value to be replicated.
    n : int
        Number of replications.

    Returns
    -------
    Replicate, out
        Replicated value.
    """
    def __init__(self, v, n):
        _Value.__init__(self)
        if not isinstance(n, int) or n < 0:
            raise TypeError("Replication count must be a positive integer")
        self.v = wrap(v)
        self.n = n


class Constant(_Value):
    """A constant, HDL-literal integer `_Value`

    Parameters
    ----------
    value : int
    bits_sign : int or tuple or None
        Either an integer `bits` or a tuple `(bits, signed)`
        specifying the number of bits in this `Constant` and whether it is
        signed (can represent negative values). `bits_sign` defaults
        to the minimum width and signedness of `value`.
    """
    def __init__(self, value, bits_sign=None):
        from migen.fhdl.bitcontainer import bits_for

        _Value.__init__(self)

        self.value = int(value)
        if bits_sign is None:
            bits_sign = bits_for(self.value), self.value < 0
        elif isinstance(bits_sign, int):
            bits_sign = bits_sign, self.value < 0
        self.nbits, self.signed = bits_sign
        if not isinstance(self.nbits, int) or self.nbits <= 0:
            raise TypeError("Width must be a strictly positive integer")

    def __hash__(self):
        return self.value


C = Constant  # shorthand


class Signal(_Value):
    """A `_Value` that can change

    The `Signal` object represents a value that is expected to change
    in the circuit. It does exactly what Verilog's `wire` and
    `reg` and VHDL's `signal` do.

    A `Signal` can be indexed to access a subset of its bits. Negative
    indices (`signal[-1]`) and the extended Python slicing notation
    (`signal[start:stop:step]`) are supported.
    The indices 0 and -1 are the least and most significant bits
    respectively.

    Parameters
    ----------
    bits_sign : int or tuple
        Either an integer `bits` or a tuple `(bits, signed)`
        specifying the number of bits in this `Signal` and whether it is
        signed (can represent negative values). `signed` defaults to
        `False`.
    name : str or None
        Name hint for this signal. If `None` (default) the name is
        inferred from the variable name this `Signal` is assigned to.
        Name collisions are automatically resolved by prepending
        names of objects that contain this `Signal` and by
        appending integer sequences.
    variable : bool
        Deprecated.
    reset : int
        Reset (synchronous) or default (combinatorial) value.
        When this `Signal` is assigned to in synchronous context and the
        corresponding clock domain is reset, the `Signal` assumes the
        given value. When this `Signal` is unassigned in combinatorial
        context (due to conditional assignments not being taken),
        the `Signal` assumes its `reset` value. Defaults to 0.
    reset_less : bool
        If `True`, do not generate reset logic for this `Signal` in
        synchronous statements. The `reset` value is only used as a
        combinatorial default or as the initial value. Defaults to `False`.
    name_override : str or None
        Do not use the inferred name but the given one.
    min : int or None
    max : int or None
        If `bits_sign` is `None`, the signal bit width and signedness are
        determined by the integer range given by `min` (inclusive,
        defaults to 0) and `max` (exclusive, defaults to 2).
    related : Signal or None
    attr : set of synthesis attributes
    """
    _name_re = _re.compile(r"^[a-zA-Z_][a-zA-Z0-9_]*$")

    def __init__(self, bits_sign=None, name=None, variable=False, reset=0,
                 reset_less=False, name_override=None, min=None, max=None,
                 related=None, attr=None):
        from migen.fhdl.bitcontainer import bits_for

        _Value.__init__(self)

        for n in [name, name_override]:
            if n is not None and not self._name_re.match(n):
                raise ValueError("Signal name {} is not a valid Python identifier"
                                 .format(repr(n)))

        # determine number of bits and signedness
        if bits_sign is None:
            if min is None:
                min = 0
            if max is None:
                max = 2
            max -= 1  # make both bounds inclusive
            assert(min < max)
            self.signed = min < 0 or max < 0
            self.nbits = _builtins.max(bits_for(min, self.signed), bits_for(max, self.signed))
        else:
            assert(min is None and max is None)
            if isinstance(bits_sign, tuple):
                self.nbits, self.signed = bits_sign
            else:
                self.nbits, self.signed = bits_sign, False
        if isinstance(reset, (bool, int)):
            reset = Constant(reset, (self.nbits, self.signed))
        if not isinstance(self.nbits, int) or self.nbits <= 0:
            raise ValueError("Signal width must be a strictly positive integer")
        if attr is None:
            attr = set()

        self.variable = variable  # deprecated
        self.reset = reset
        self.reset_less = reset_less
        self.name_override = name_override
        self.backtrace = _tracer.trace_back(name)
        self.related = related
        self.attr = attr

    def __setattr__(self, k, v):
        if k == "reset":
            v = wrap(v)
        _Value.__setattr__(self, k, v)

    def __repr__(self):
        return "<Signal " + (self.backtrace[-1][0] or "anonymous") + " at " + hex(id(self)) + ">"

    @classmethod
    def like(cls, other, **kwargs):
        """Create Signal based on another.

        Parameters
        ----------
        other : _Value
            Object to base this Signal on.

        See `migen.fhdl.bitcontainer.value_bits_sign` for details.
        """
        from migen.fhdl.bitcontainer import value_bits_sign
        kw = dict(bits_sign=value_bits_sign(other))
        if isinstance(other, cls):
            kw.update(variable=other.variable,
                    reset=other.reset.value, reset_less=other.reset_less,
                    related=other.related, attr=set(other.attr))
        kw.update(kwargs)
        return cls(**kw)

    def __hash__(self):
        return self.duid


class ClockSignal(_Value):
    """Clock signal for a given clock domain

    `ClockSignal` s for a given clock domain can be retrieved multiple
    times. They all ultimately refer to the same signal.

    Parameters
    ----------
    cd : str
        Clock domain to obtain a clock signal for. Defaults to `"sys"`.
    """
    def __init__(self, cd="sys"):
        _Value.__init__(self)
        if not isinstance(cd, str):
            raise TypeError("Argument of ClockSignal must be a string")
        self.cd = cd


class ResetSignal(_Value):
    """Reset signal for a given clock domain

    `ResetSignal` s for a given clock domain can be retrieved multiple
    times. They all ultimately refer to the same signal.

    Parameters
    ----------
    cd : str
        Clock domain to obtain a reset signal for. Defaults to `"sys"`.
    allow_reset_less : bool
        If the clock domain is resetless, return 0 instead of reporting an
        error.
    """
    def __init__(self, cd="sys", allow_reset_less=False):
        _Value.__init__(self)
        if not isinstance(cd, str):
            raise TypeError("Argument of ResetSignal must be a string")
        self.cd = cd
        self.allow_reset_less = allow_reset_less


# statements


class _Statement:
    pass


class _Assign(_Statement):
    def __init__(self, l, r):
        self.l = wrap(l)
        self.r = wrap(r)


def _check_statement(s):
    if isinstance(s, _collections_abc.Iterable):
        return all(_check_statement(ss) for ss in s)
    else:
        return isinstance(s, _Statement)


class If(_Statement):
    """Conditional execution of statements

    Parameters
    ----------
    cond : _Value(1), in
        Condition
    *t : Statements
        Statements to execute if `cond` is asserted.

    Examples
    --------
    >>> a = Signal()
    >>> b = Signal()
    >>> c = Signal()
    >>> d = Signal()
    >>> If(a,
    ...     b.eq(1)
    ... ).Elif(c,
    ...     b.eq(0)
    ... ).Else(
    ...     b.eq(d)
    ... )
    """
    def __init__(self, cond, *t):
        if not _check_statement(t):
            raise TypeError("Not all test body objects are Migen statements")
        self.cond = wrap(cond)
        self.t = list(t)
        self.f = []

    def Else(self, *f):
        """Add an `else` conditional block

        Parameters
        ----------
        *f : Statements
            Statements to execute if all previous conditions fail.
        """
        if not _check_statement(f):
            raise TypeError("Not all test body objects are Migen statements")
        _insert_else(self, list(f))
        return self

    def Elif(self, cond, *t):
        """Add an `else if` conditional block

        Parameters
        ----------
        cond : _Value(1), in
            Condition
        *t : Statements
            Statements to execute if previous conditions fail and `cond`
            is asserted.
        """
        _insert_else(self, [If(cond, *t)])
        return self


def _insert_else(obj, clause):
    o = obj
    while o.f:
        assert(len(o.f) == 1)
        assert(isinstance(o.f[0], If))
        o = o.f[0]
    o.f = clause


class Case(_Statement):
    """Case/Switch statement

    Parameters
    ----------
    test : _Value, in
        Selector value used to decide which block to execute
    cases : dict
        Dictionary of cases. The keys are numeric constants to compare
        with `test`. The values are statements to be executed the
        corresponding key matches `test`. The dictionary may contain a
        string key `"default"` to mark a fall-through case that is
        executed if no other key matches.

    Examples
    --------
    >>> a = Signal()
    >>> b = Signal()
    >>> Case(a, {
    ...     0:         b.eq(1),
    ...     1:         b.eq(0),
    ...     "default": b.eq(0),
    ... })
    """
    def __init__(self, test, cases):
        self.test = wrap(test)
        self.cases = dict()
        for k, v in cases.items():
            if isinstance(k, (bool, int)):
                k = Constant(k)
            if (not isinstance(k, Constant)
                    and not (isinstance(k, str) and k == "default")):
                raise TypeError("Case object is not a Migen constant")
            if not isinstance(v, _collections_abc.Iterable):
                v = [v]
            if not _check_statement(v):
                raise TypeError("Not all objects for case {} "
                                "are Migen statements".format(k))
            self.cases[k] = v

    def makedefault(self, key=None):
        """Mark a key as the default case

        Deletes/substitutes any previously existing default case.

        Parameters
        ----------
        key : int, Constant or None
            Key to use as default case if no other key matches.
            By default, the largest key is the default key.
        """
        if key is None:
            for choice in self.cases.keys():
                if (key is None
                        or (isinstance(choice, str) and choice == "default")
                        or choice.value > key.value):
                    key = choice
        if not isinstance(key, str) or key != "default":
            key = wrap(key)
        stmts = self.cases[key]
        del self.cases[key]
        self.cases["default"] = stmts
        return self


# arrays


class _ArrayProxy(_Value):
    def __init__(self, choices, key):
        _Value.__init__(self)
        self.choices = []
        for c in choices:
            if isinstance(c, (bool, int)):
                c = Constant(c)
            self.choices.append(c)
        self.key = key

    def __getattr__(self, attr):
        return _ArrayProxy([getattr(choice, attr) for choice in self.choices],
            self.key)

    def __getitem__(self, key):
        return _ArrayProxy([choice.__getitem__(key) for choice in self.choices],
            self.key)


class Array(list):
    """Addressable multiplexer

    An array is created from an iterable of values and indexed using the
    usual Python simple indexing notation (no negative indices or
    slices). It can be indexed by numeric constants, `_Value` s, or
    `Signal` s.

    The result of indexing the array is a proxy for the entry at the
    given index that can be used on either RHS or LHS of assignments.

    An array can be indexed multiple times.

    Multidimensional arrays are supported by packing inner arrays into
    outer arrays.

    Parameters
    ----------
    values : iterable of ints, _Values, Signals
        Entries of the array. Each entry can be a numeric constant, a
        `Signal` or a `Record`.

    Examples
    --------
    >>> a = Array(range(10))
    >>> b = Signal(max=10)
    >>> c = Signal(max=10)
    >>> b.eq(a[9 - c])
    """
    def __getitem__(self, key):
        if isinstance(key, Constant):
            return list.__getitem__(self, key.value)
        elif isinstance(key, _Value):
            return _ArrayProxy(self, key)
        else:
            return list.__getitem__(self, key)

# clock domains

class ClockDomain:
    """Synchronous domain

    Parameters
    ----------
    name : str or None
        Domain name. If None (the default) the name is inferred from the
        variable name this `ClockDomain` is assigned to (stripping any
        `"cd_"` prefix).
    reset_less : bool
        The domain does not use a reset signal. Registers within this
        domain are still all initialized to their reset state once, e.g.
        through Verilog `"initial"` statements.

    Attributes
    ----------
    clk : Signal, inout
        The clock for this domain. Can be driven or used to drive other
        signals (preferably in combinatorial context).
    rst : Signal or None, inout
        Reset signal for this domain. Can be driven or used to drive.
    """
    def __init__(self, name=None, reset_less=False):
        self.name = _tracer.get_obj_var_name(name)
        if self.name is None:
            raise ValueError("Cannot extract clock domain name from code, need to specify.")
        if self.name.startswith("cd_"):
            self.name = self.name[3:]
        if self.name[0].isdigit():
            raise ValueError("Clock domain name cannot start with a number.")
        self.clk = Signal(name_override=self.name + "_clk")
        if reset_less:
            self.rst = None
        else:
            self.rst = Signal(name_override=self.name + "_rst")

    def rename(self, new_name):
        """Rename the clock domain

        Parameters
        ----------
        new_name : str
            New name
        """
        self.name = new_name
        self.clk.name_override = new_name + "_clk"
        if self.rst is not None:
            self.rst.name_override = new_name + "_rst"


class _ClockDomainList(list):
    def __getitem__(self, key):
        if isinstance(key, str):
            for cd in self:
                if cd.name == key:
                    return cd
            raise KeyError(key)
        else:
            return list.__getitem__(self, key)

    def __contains__(self, cd_or_name):
        if isinstance(cd_or_name, str):
            for cd in self:
                if cd.name == cd_or_name:
                    return True
            return False
        else:
            return list.__contains__(self, cd_or_name)


# specials

(SPECIAL_INPUT, SPECIAL_OUTPUT, SPECIAL_INOUT) = range(3)

# simulation

class Display(_Statement):
    def __init__(self, s, *args):
        self.s = s
        self.args = args

class Finish(_Statement):
    pass

# fragment

class _Fragment:
    def __init__(self, comb=None, sync=None, specials=None, clock_domains=None):
        if comb is None: comb = []
        if sync is None: sync = dict()
        if specials is None: specials = set()
        if clock_domains is None: clock_domains = _ClockDomainList()

        self.comb = comb
        self.sync = sync
        self.specials = specials
        self.clock_domains = _ClockDomainList(clock_domains)

    def __add__(self, other):
        newsync = _collections.defaultdict(list)
        for k, v in self.sync.items():
            newsync[k] = v[:]
        for k, v in other.sync.items():
            newsync[k].extend(v)
        return _Fragment(self.comb + other.comb, newsync,
            self.specials | other.specials,
            self.clock_domains + other.clock_domains)

    def __iadd__(self, other):
        newsync = _collections.defaultdict(list)
        for k, v in self.sync.items():
            newsync[k] = v[:]
        for k, v in other.sync.items():
            newsync[k].extend(v)
        self.comb += other.comb
        self.sync = newsync
        self.specials |= other.specials
        self.clock_domains += other.clock_domains
        return self



from migen.build.generic_platform import *
from migen.build.xilinx import XilinxPlatform


_io = [
    ("clk50", 0, Pins("E15"), IOStandard("LVCMOS33")),

    ("serial", 0,
        Subsignal("tx", Pins("C16")),
        Subsignal("rx", Pins("B17")),
        IOStandard("LVCMOS33")
    ),

    ("amc_rtm_serwb", 0,
        Subsignal("clk", Pins("R18")), # rtm_fpga_usr_io_p
        Subsignal("tx", Pins("T17")),  # rtm_fpga_lvds2_p
        Subsignal("rx", Pins("R16")),  # rtm_fpga_lvds1_p
        IOStandard("LVCMOS18")
    ),

    # HMC clocking chips (830 and 7043)
    ("hmc_spi", 0,
        Subsignal("clk", Pins("A17"), Misc("PULLDOWN=TRUE")),
        # cs[0]=830 cs[1]=7043
        # Watch out for the HMC830 SPI mode peculiarity. PULLDOWN CS here
        # so that toggling the SPI core offline will make edges.
        Subsignal("cs_n", Pins("C8 D16"), Misc("PULLDOWN=TRUE")),
        Subsignal("mosi", Pins("B16"), Misc("PULLDOWN=TRUE")),
        Subsignal("miso", Pins("D9"), Misc("PULLDOWN=TRUE")),
        IOStandard("LVCMOS33")
    ),
    # Shared with Si5324 reset. Among many Sayma bugs, those resets
    # have opposite polarities, so both chips cannot be used at the
    # same time.
    ("hmc7043_reset", 0, Pins("E17"), IOStandard("LVCMOS33")),
    ("hmc7043_gpo", 0, Pins("D8"), IOStandard("LVCMOS33")),

    # clock mux
    ("clk_src_ext_sel", 0, Pins("P15"), IOStandard("LVCMOS18")),
    ("ref_clk_src_sel", 0, Pins("J14"), IOStandard("LVCMOS18")),
    ("dac_clk_src_sel", 0, Pins("P16"), IOStandard("LVCMOS18")),
    ("ref_lo_clk_sel", 0, Pins("L18"), IOStandard("LVCMOS18")),

    # DACs
    ("ad9154_rst_n", 0, Pins("U15"), IOStandard("LVCMOS18")),
    ("ad9154_spi", 0,
        Subsignal("clk", Pins("T13"), Misc("PULLDOWN=TRUE")),
        Subsignal("cs_n", Pins("U14"), Misc("PULLUP=TRUE")),
        Subsignal("mosi", Pins("V17"), Misc("PULLDOWN=TRUE")),
        Subsignal("miso", Pins("R13"), Misc("PULLDOWN=TRUE")),
        IOStandard("LVCMOS18")
    ),
    ("ad9154_txen", 0, Pins("V16 U16"), IOStandard("LVCMOS18")),
    ("ad9154_spi", 1,
        Subsignal("clk", Pins("J15"), Misc("PULLDOWN=TRUE")),
        Subsignal("cs_n", Pins("K18"), Misc("PULLUP=TRUE")),
        Subsignal("mosi", Pins("J18"), Misc("PULLDOWN=TRUE")),
        Subsignal("miso", Pins("J16"), Misc("PULLDOWN=TRUE")),
        IOStandard("LVCMOS18")
    ),
    ("ad9154_txen", 1, Pins("L17 L14"), IOStandard("LVCMOS18")),

    # Allaki
    ("allaki0_rfsw0", 0, Pins("R2"), IOStandard("LVCMOS33")),
    ("allaki0_rfsw1", 0, Pins("N4"), IOStandard("LVCMOS33")),
    ("allaki1_rfsw0", 0, Pins("R5"), IOStandard("LVCMOS33")),
    ("allaki1_rfsw1", 0, Pins("V4"), IOStandard("LVCMOS33")),
    ("allaki2_rfsw0", 0, Pins("B15"), IOStandard("LVCMOS33")),
    ("allaki2_rfsw1", 0, Pins("B12"), IOStandard("LVCMOS33")),
    ("allaki3_rfsw0", 0, Pins("L4"), IOStandard("LVCMOS33")),
    ("allaki3_rfsw1", 0, Pins("J4"), IOStandard("LVCMOS33")),

    ("allaki0_att0", 0,
        Subsignal("le", Pins("U2")),
        Subsignal("sin", Pins("T4")),
        Subsignal("clk", Pins("V3")),
        Subsignal("rst_n", Pins("R3")),
        IOStandard("LVCMOS33")
    ),
    ("allaki0_att1", 0,
        Subsignal("le", Pins("P1")),
        Subsignal("sin", Pins("P3")),
        Subsignal("clk", Pins("N3")),
        Subsignal("rst_n", Pins("M4")),
        IOStandard("LVCMOS33")
    ),
    ("allaki1_att0", 0,
        Subsignal("le", Pins("U7")),
        Subsignal("sin", Pins("R6")),
        Subsignal("clk", Pins("V8")),
        Subsignal("rst_n", Pins("R7")),
        IOStandard("LVCMOS33")
    ),
    ("allaki1_att1", 0,
        Subsignal("le", Pins("T3")),
        Subsignal("sin", Pins("U5")),
        Subsignal("clk", Pins("P6")),
        Subsignal("rst_n", Pins("U4")),
        IOStandard("LVCMOS33")
    ),
    ("allaki2_att0", 0,
        Subsignal("le", Pins("C13")),
        Subsignal("sin", Pins("E16")),
        Subsignal("clk", Pins("D14")),
        Subsignal("rst_n", Pins("A15")),
        IOStandard("LVCMOS33")
    ),
    ("allaki2_att1", 0,
        Subsignal("le", Pins("D11")),
        Subsignal("sin", Pins("C14")),
        Subsignal("clk", Pins("A12")),
        Subsignal("rst_n", Pins("C12")),
        IOStandard("LVCMOS33")
    ),
    ("allaki3_att0", 0,
        Subsignal("le", Pins("M2")),
        Subsignal("sin", Pins("N1")),
        Subsignal("clk", Pins("M6")),
        Subsignal("rst_n", Pins("L5")),
        IOStandard("LVCMOS33")
    ),
    ("allaki3_att1", 0,
        Subsignal("le", Pins("K5")),
        Subsignal("sin", Pins("L2")),
        Subsignal("clk", Pins("K2")),
        Subsignal("rst_n", Pins("J5")),
        IOStandard("LVCMOS33")
    ),

    ("i2c", 0,
        Subsignal("scl", Pins("J6")),
        Subsignal("sda", Pins("K6")),
        IOStandard("LVCMOS33")
    ),

    ("si5324_clkin", 0,
        Subsignal("p", Pins("M16")),
        Subsignal("n", Pins("M17")),
        IOStandard("DIFF_SSTL18_I"),
    ),
    ("si5324_clkout", 0,
        Subsignal("p", Pins("B6")),
        Subsignal("n", Pins("B5"))
    ),
    ("si5324_clkout_fabric", 0,
        Subsignal("p", Pins("T14")),
        Subsignal("n", Pins("T15")),
        IOStandard("LVDS_25")
    ),
    ("rtm_master_aux_clk", 0,
        Subsignal("p", Pins("P14")),
        Subsignal("n", Pins("R15")),
        IOStandard("LVDS_25")
    ),
    # Slave SATA connector J71
    ("sata", 0,
        Subsignal("txp", Pins("D2")),
        Subsignal("txn", Pins("D1")),
        Subsignal("rxp", Pins("C4")),
        Subsignal("rxn", Pins("C3"))
    ),
]

_connectors = [
    ("clk_mez", {
        "gpio_{}".format(num): pin for num, pin in enumerate(
             "D18 C17 C18 G17 F18 H16 G15 G15"
             "F15 G14 F14 H17 H18 F17 H14 E18".split())}),
]


class Platform(XilinxPlatform):
    default_clk_name = "clk50"
    default_clk_period = 20.0

    def __init__(self, larger=False):
        chip = "xc7a50t-csg325-1" if larger else "xc7a15t-csg325-1"
        XilinxPlatform.__init__(self, chip, _io, _connectors,
                                toolchain="vivado")
        self.toolchain.bitstream_commands.extend([
            # FIXME: enable this when the XADC reference wiring is fixed
            # "set_property BITSTREAM.CONFIG.OVERTEMPPOWERDOWN Enable [current_design]",
            "set_property BITSTREAM.GENERAL.COMPRESS True [current_design]",
            "set_property CFGBVS VCCO [current_design]",
            "set_property CONFIG_VOLTAGE 3.3 [current_design]",
        ])



from copy import copy
from operator import itemgetter

from migen.fhdl.structure import *
from migen.fhdl.structure import (_Operator, _Slice, _Part, _Assign, _ArrayProxy,
                                  _Fragment)


class NodeVisitor:
    def visit(self, node):
        if isinstance(node, Constant):
            self.visit_Constant(node)
        elif isinstance(node, Signal):
            self.visit_Signal(node)
        elif isinstance(node, ClockSignal):
            self.visit_ClockSignal(node)
        elif isinstance(node, ResetSignal):
            self.visit_ResetSignal(node)
        elif isinstance(node, _Operator):
            self.visit_Operator(node)
        elif isinstance(node, _Slice):
            self.visit_Slice(node)
        elif isinstance(node, Cat):
            self.visit_Cat(node)
        elif isinstance(node, Replicate):
            self.visit_Replicate(node)
        elif isinstance(node, _Assign):
            self.visit_Assign(node)
        elif isinstance(node, If):
            self.visit_If(node)
        elif isinstance(node, Case):
            self.visit_Case(node)
        elif isinstance(node, _Fragment):
            self.visit_Fragment(node)
        elif isinstance(node, (list, tuple)):
            self.visit_statements(node)
        elif isinstance(node, dict):
            self.visit_clock_domains(node)
        elif isinstance(node, _ArrayProxy):
            self.visit_ArrayProxy(node)
        else:
            self.visit_unknown(node)

    def visit_Constant(self, node):
        pass

    def visit_Signal(self, node):
        pass

    def visit_ClockSignal(self, node):
        pass

    def visit_ResetSignal(self, node):
        pass

    def visit_Operator(self, node):
        for o in node.operands:
            self.visit(o)

    def visit_Slice(self, node):
        self.visit(node.value)

    def visit_Part(self, node):
        self.visit(node.value)
        self.visit(node.offset)

    def visit_Cat(self, node):
        for e in node.l:
            self.visit(e)

    def visit_Replicate(self, node):
        self.visit(node.v)

    def visit_Assign(self, node):
        self.visit(node.l)
        self.visit(node.r)

    def visit_If(self, node):
        self.visit(node.cond)
        self.visit(node.t)
        self.visit(node.f)

    def visit_Case(self, node):
        self.visit(node.test)
        for v, statements in sorted(node.cases.items(),
                                    key=lambda x: -1 if x[0] is "default" else x[0].duid):
            self.visit(statements)

    def visit_Fragment(self, node):
        self.visit(node.comb)
        self.visit(node.sync)

    def visit_statements(self, node):
        for statement in node:
            self.visit(statement)

    def visit_clock_domains(self, node):
        for clockname, statements in sorted(node.items(), key=itemgetter(0)):
            self.visit(statements)

    def visit_ArrayProxy(self, node):
        for choice in node.choices:
            self.visit(choice)
        self.visit(node.key)

    def visit_unknown(self, node):
        pass


# Default methods always copy the node, except for:
# - Signals, ClockSignals and ResetSignals
# - Unknown objects
# - All fragment fields except comb and sync
# In those cases, the original node is returned unchanged.
class NodeTransformer:
    def visit(self, node):
        if isinstance(node, Constant):
            return self.visit_Constant(node)
        elif isinstance(node, Signal):
            return self.visit_Signal(node)
        elif isinstance(node, ClockSignal):
            return self.visit_ClockSignal(node)
        elif isinstance(node, ResetSignal):
            return self.visit_ResetSignal(node)
        elif isinstance(node, _Operator):
            return self.visit_Operator(node)
        elif isinstance(node, _Slice):
            return self.visit_Slice(node)
        elif isinstance(node, _Part):
            return self.visit_Part(node)
        elif isinstance(node, Cat):
            return self.visit_Cat(node)
        elif isinstance(node, Replicate):
            return self.visit_Replicate(node)
        elif isinstance(node, _Assign):
            return self.visit_Assign(node)
        elif isinstance(node, If):
            return self.visit_If(node)
        elif isinstance(node, Case):
            return self.visit_Case(node)
        elif isinstance(node, _Fragment):
            return self.visit_Fragment(node)
        elif isinstance(node, (list, tuple)):
            return self.visit_statements(node)
        elif isinstance(node, dict):
            return self.visit_clock_domains(node)
        elif isinstance(node, _ArrayProxy):
            return self.visit_ArrayProxy(node)
        else:
            return self.visit_unknown(node)

    def visit_Constant(self, node):
        return node

    def visit_Signal(self, node):
        return node

    def visit_ClockSignal(self, node):
        return node

    def visit_ResetSignal(self, node):
        return node

    def visit_Operator(self, node):
        return _Operator(node.op, [self.visit(o) for o in node.operands])

    def visit_Slice(self, node):
        return _Slice(self.visit(node.value), node.start, node.stop)

    def visit_Part(self, node):
        return _Part(self.visit(node.value), self.visit(node.offset), node.width)

    def visit_Cat(self, node):
        return Cat(*[self.visit(e) for e in node.l])

    def visit_Replicate(self, node):
        return Replicate(self.visit(node.v), node.n)

    def visit_Assign(self, node):
        return _Assign(self.visit(node.l), self.visit(node.r))

    def visit_If(self, node):
        r = If(self.visit(node.cond))
        r.t = self.visit(node.t)
        r.f = self.visit(node.f)
        return r

    def visit_Case(self, node):
        cases = {v: self.visit(statements)
                 for v, statements in sorted(node.cases.items(),
                                             key=lambda x: -1 if x[0] is "default" else x[0].duid)}
        r = Case(self.visit(node.test), cases)
        return r

    def visit_Fragment(self, node):
        r = copy(node)
        r.comb = self.visit(node.comb)
        r.sync = self.visit(node.sync)
        return r

    # NOTE: this will always return a list, even if node is a tuple
    def visit_statements(self, node):
        return [self.visit(statement) for statement in node]

    def visit_clock_domains(self, node):
        return {clockname: self.visit(statements)
            for clockname, statements in sorted(node.items(),
                                                key=itemgetter(0))}

    def visit_ArrayProxy(self, node):
        return _ArrayProxy([self.visit(choice) for choice in node.choices],
            self.visit(node.key))

    def visit_unknown(self, node):
        return node



from migen.build.generic_platform import *
from migen.build.lattice import LatticePlatform
from migen.build.lattice.programmer import IceStormProgrammer


_io = [
    ("user_led", 0, Pins("99"), IOStandard("LVCMOS33")),
    ("user_led", 1, Pins("98"), IOStandard("LVCMOS33")),
    ("user_led", 2, Pins("97"), IOStandard("LVCMOS33")),
    ("user_led", 3, Pins("96"), IOStandard("LVCMOS33")),
    ("user_led", 4, Pins("95"), IOStandard("LVCMOS33")),

    ("serial", 0,
        Subsignal("rx", Pins("9")),
        Subsignal("tx", Pins("8"), Misc("PULLUP")),
        Subsignal("rts", Pins("7"), Misc("PULLUP")),
        Subsignal("cts", Pins("4"), Misc("PULLUP")),
        Subsignal("dtr", Pins("3"), Misc("PULLUP")),
        Subsignal("dsr", Pins("2"), Misc("PULLUP")),
        Subsignal("dcd", Pins("1"), Misc("PULLUP")),
        IOStandard("LVTTL"),
    ),

    ("irda", 0,
        Subsignal("rx", Pins("106")),
        Subsignal("tx", Pins("105")),
        Subsignal("sd", Pins("107")),
        IOStandard("LVCMOS33")
    ),

    ("spiflash", 0,
        Subsignal("cs_n", Pins("71"), IOStandard("LVCMOS33")),
        Subsignal("clk", Pins("70"), IOStandard("LVCMOS33")),
        Subsignal("mosi", Pins("67"), IOStandard("LVCMOS33")),
        Subsignal("miso", Pins("68"), IOStandard("LVCMOS33"))
    ),

    ("clk12", 0, Pins("21"), IOStandard("LVCMOS33"))
]

_connectors = [
    ("GPIO0", "44 45 47 48 56 60 61 62"),
    ("GPIO1", "119 118 117 116 115 114 113 112"),
    ("PMOD", "78 79 80 81 87 88 90 91")
]


class Platform(LatticePlatform):
    default_clk_name = "clk12"
    default_clk_period = 83.333

    def __init__(self):
        LatticePlatform.__init__(self, "ice40-hx1k-tq144", _io, _connectors,
                                 toolchain="icestorm")

    def create_programmer(self):
        return IceStormProgrammer()



import os
import struct
import re
import subprocess
import sys
import ctypes


def language_by_filename(name):
    extension = name.rsplit(".")[-1]
    if extension in ["v", "vh", "vo"]:
        return "verilog"
    if extension in ["vhd", "vhdl", "vho"]:
        return "vhdl"
    return None


def write_to_file(filename, contents, force_unix=False):
    newline = None
    if force_unix:
        newline = "\n"
    with open(filename, "w", newline=newline) as f:
        f.write(contents)


def sub_rules(line, rules, max_matches=1):
    for pattern, color in rules:
        line, matches = re.subn(pattern, color, line, max_matches)
        max_matches -= matches
        if not max_matches:
            break
    return line


def subprocess_call_filtered(command, rules, *, max_matches=1, **kwargs):
    with subprocess.Popen(command, stdout=subprocess.PIPE,
                          universal_newlines=True, bufsize=1,
                          **kwargs) as proc:
        with open(proc.stdout.fileno(), errors="ignore", closefd=False) as stdout:
            for line in stdout:
                print(sub_rules(line, rules, max_matches), end="")
        return proc.wait()


if sys.platform == "cygwin":
    cygwin1 = ctypes.CDLL("/usr/bin/cygwin1.dll")
    cygwin_conv_path_proto = ctypes.CFUNCTYPE(
        ctypes.c_ssize_t, # Return
        ctypes.c_uint, # what
        ctypes.c_void_p, # from
        ctypes.c_void_p, # to
        ctypes.c_size_t) # size
    cygwin_conv_path = cygwin_conv_path_proto(("cygwin_conv_path", cygwin1),
        ((1, "what"),
        (1, "from"),
        (1, "to"),
        (1, "size")))


    def cygpath_to_windows(path):
        what = ctypes.c_uint(0) # CCP_POSIX_TO_WIN_A
        fro = ctypes.c_char_p(path.encode('utf-8'))
        to = ctypes.byref(ctypes.create_string_buffer(260))
        size = ctypes.c_size_t(260)

        cygwin_conv_path(what, fro, to, size)
        return ctypes.cast(to, ctypes.c_char_p).value.decode('utf-8')

    # Convert cygwin paths to Windows native paths. This is a noop otherwise.
    def cygpath(p):
        return cygpath_to_windows(p)
else:
    def cygpath(p):
        return p



from migen import *
from migen.build.generic_platform import *
from migen.build.lattice import LatticePlatform
from migen.build.lattice.programmer import IceStormProgrammer


_io = [
    ("rgb_led", 0,
        Subsignal("r", Pins("41")),
        Subsignal("g", Pins("40")),
        Subsignal("b", Pins("39")),
        IOStandard("LVCMOS33")
    ),
]

spiflash = [
    # Only usable in PROG FLASH mode - see JP2 header 
    ("spiflash", 0,
        Subsignal("cs_n", Pins("16"), IOStandard("LVCMOS33")),
        Subsignal("clk", Pins("15"), IOStandard("LVCMOS33")),
        Subsignal("mosi", Pins("14"), IOStandard("LVCMOS33")),
        Subsignal("miso", Pins("17"), IOStandard("LVCMOS33")),
    ),
]


_connectors = [

    # JP5's pinout is all Free, except 1 (3.3V) and 2 (GND).
    #         3  4  5  6  7  8  9 10 11 12 13 14 15 16
    ("JP5", "23 25 26 27 32 35 31 37 34 43 36 42 38 28"),
    
    #         1  2  3  4  5  6  7  8  9 10 11 12 13 14 15 16
    ("JP6", "12 21 13 19 18 11  9  6 44  4  3 48 45 47 46  2"),
]

class MachClock(Module):
    def __init__(self, period, out):
        self.specials += Instance("SB_HFOSC",
                                  i_CLKHFPU=C(1),
                                  i_CLKHFEN=C(1),
                                  o_CLKHF=out
                                  )

class HfoscRouting(Module):
    def __init__(self):
        self.hfosc_used = False  # Only one default clock,
        self.mach_clk_sig = Signal()

    def mk_clk(self, name, clk_period):
        if not self.hfosc_used:
            self.mach_clk_sig.name_override = name
            self.submodules.mclk = MachClock(clk_period, self.mach_clk_sig)
            self.hfosc_used = True
        else:
            raise ConstraintError
        return self.mach_clk_sig

class Platform(LatticePlatform):
    default_clk_name = "sb_hfosc"
    default_clk_period = 48
    
    def __init__(self):
        self.sb_hfosc_routing = HfoscRouting()    # Internal oscillator routing.
        LatticePlatform.__init__(self, "ice40-up5k-sg48", _io, _connectors,
                                 toolchain="icestorm")
        
    def request(self, *args, **kwargs):
        try:
            sig = GenericPlatform.request(self, *args, **kwargs)
        except ConstraintError:
            # ICE40UP5K  internal clock
            if args[0] == "sb_hfosc":
                # Do not add to self.constraint_manager.matched because we
                # don't want this signal to become part of the UCF.
                sig = self.sb_hfosc_routing.mk_clk("sb_hfosc", 48)
        return sig

    def do_finalize(self, f, *args, **kwargs):
        f += self.sb_hfosc_routing.get_fragment()

        # Handle cases where hfosc is default not default.
        if self.default_clk_name != "sb_hfosc":
            GenericPlatform.do_finalize(self, f, *args, **kwargs)

        if self.default_clk_name == "sb_hfosc":
            self.default_clk_period = 48



from collections import OrderedDict
from itertools import combinations

from migen.fhdl.structure import *


class _Node:
    def __init__(self):
        self.signal_count = 0
        self.numbers = set()
        self.use_name = False
        self.use_number = False
        self.children = OrderedDict()


def _display_tree(filename, tree):
    from migen.util.treeviz import RenderNode

    def _to_render_node(name, node):
        children = [_to_render_node(k, v) for k, v in node.children.items()]
        if node.use_name:
            if node.use_number:
                color = (0.5, 0.9, 0.8)
            else:
                color = (0.8, 0.5, 0.9)
        else:
            if node.use_number:
                color = (0.9, 0.8, 0.5)
            else:
                color = (0.8, 0.8, 0.8)
        label = "{0}\n{1} signals\n{2}".format(name, node.signal_count, node.numbers)
        return RenderNode(label, children, color=color)

    top = _to_render_node("top", tree)
    top.to_svg(filename)


def _build_tree(signals, basic_tree=None):
    root = _Node()
    for signal in signals:
        current_b = basic_tree
        current = root
        current.signal_count += 1
        for name, number in signal.backtrace:
            if basic_tree is None:
                use_number = False
            else:
                current_b = current_b.children[name]
                use_number = current_b.use_number
            if use_number:
                key = (name, number)
            else:
                key = name
            try:
                current = current.children[key]
            except KeyError:
                new = _Node()
                current.children[key] = new
                current = new
            current.numbers.add(number)
            if use_number:
                current.all_numbers = sorted(current_b.numbers)
            current.signal_count += 1
    return root


def _set_use_name(node, node_name=""):
    cnames = [(k, _set_use_name(v, k)) for k, v in node.children.items()]
    for (c1_prefix, c1_names), (c2_prefix, c2_names) in combinations(cnames, 2):
        if not c1_names.isdisjoint(c2_names):
            node.children[c1_prefix].use_name = True
            node.children[c2_prefix].use_name = True
    r = set()
    for c_prefix, c_names in cnames:
        if node.children[c_prefix].use_name:
            for c_name in c_names:
                r.add((c_prefix, ) + c_name)
        else:
            r |= c_names

    if node.signal_count > sum(c.signal_count for c in node.children.values()):
        node.use_name = True
        r.add((node_name, ))

    return r


def _name_signal(tree, signal):
    elements = []
    treepos = tree
    for step_name, step_n in signal.backtrace:
        try:
            treepos = treepos.children[(step_name, step_n)]
            use_number = True
        except KeyError:
            treepos = treepos.children[step_name]
            use_number = False
        if treepos.use_name:
            elname = step_name
            if use_number:
                elname += str(treepos.all_numbers.index(step_n))
            elements.append(elname)
    return "_".join(elements)


def _build_pnd_from_tree(tree, signals):
    return dict((signal, _name_signal(tree, signal)) for signal in signals)


def _invert_pnd(pnd):
    inv_pnd = dict()
    for k, v in pnd.items():
        inv_pnd[v] = inv_pnd.get(v, [])
        inv_pnd[v].append(k)
    return inv_pnd


def _list_conflicting_signals(pnd):
    inv_pnd = _invert_pnd(pnd)
    r = set()
    for k, v in inv_pnd.items():
        if len(v) > 1:
            r.update(v)
    return r


def _set_use_number(tree, signals):
    for signal in signals:
        current = tree
        for step_name, step_n in signal.backtrace:
            current = current.children[step_name]
            current.use_number = current.signal_count > len(current.numbers) and len(current.numbers) > 1

_debug = False


def _build_pnd_for_group(group_n, signals):
    basic_tree = _build_tree(signals)
    _set_use_name(basic_tree)
    if _debug:
        _display_tree("tree{0}_basic.svg".format(group_n), basic_tree)
    pnd = _build_pnd_from_tree(basic_tree, signals)

    # If there are conflicts, try splitting the tree by numbers
    # on paths taken by conflicting signals.
    conflicting_signals = _list_conflicting_signals(pnd)
    if conflicting_signals:
        _set_use_number(basic_tree, conflicting_signals)
        if _debug:
            print("namer: using split-by-number strategy (group {0})".format(group_n))
            _display_tree("tree{0}_marked.svg".format(group_n), basic_tree)
        numbered_tree = _build_tree(signals, basic_tree)
        _set_use_name(numbered_tree)
        if _debug:
            _display_tree("tree{0}_numbered.svg".format(group_n), numbered_tree)
        pnd = _build_pnd_from_tree(numbered_tree, signals)
    else:
        if _debug:
            print("namer: using basic strategy (group {0})".format(group_n))

    # ...then add number suffixes by DUID
    inv_pnd = _invert_pnd(pnd)
    duid_suffixed = False
    for name, signals in inv_pnd.items():
        if len(signals) > 1:
            duid_suffixed = True
            for n, signal in enumerate(sorted(signals, key=lambda x: x.duid)):
                pnd[signal] += str(n)
    if _debug and duid_suffixed:
        print("namer: using DUID suffixes (group {0})".format(group_n))

    return pnd


def _build_signal_groups(signals):
    r = []
    for signal in signals:
        # build chain of related signals
        related_list = []
        cur_signal = signal
        while cur_signal is not None:
            related_list.insert(0, cur_signal)
            cur_signal = cur_signal.related
        # add to groups
        for _ in range(len(related_list) - len(r)):
            r.append(set())
        for target_set, source_signal in zip(r, related_list):
            target_set.add(source_signal)
    # with the algorithm above and a list of all signals,
    # a signal appears in all groups of a lower number than its.
    # make signals appear only in their group of highest number.
    for s1, s2 in zip(r, r[1:]):
        s1 -= s2
    return r


def _build_pnd(signals):
    groups = _build_signal_groups(signals)
    gpnds = [_build_pnd_for_group(n, gsignals) for n, gsignals in enumerate(groups)]

    pnd = dict()
    for gn, gpnd in enumerate(gpnds):
        for signal, name in gpnd.items():
            result = name
            cur_gn = gn
            cur_signal = signal
            while cur_signal.related is not None:
                cur_signal = cur_signal.related
                cur_gn -= 1
                result = gpnds[cur_gn][cur_signal] + "_" + result
            pnd[signal] = result

    return pnd


def build_namespace(signals, reserved_keywords=set()):
    pnd = _build_pnd(signals)
    ns = Namespace(pnd, reserved_keywords)
    # register signals with name_override
    swno = {signal for signal in signals if signal.name_override is not None}
    for signal in sorted(swno, key=lambda x: x.duid):
        ns.get_name(signal)
    return ns


class Namespace:
    def __init__(self, pnd, reserved_keywords=set()):
        self.counts = {k: 1 for k in reserved_keywords}
        self.sigs = {}
        self.pnd = pnd
        self.clock_domains = dict()

    def get_name(self, sig):
        if isinstance(sig, ClockSignal):
            sig = self.clock_domains[sig.cd].clk
        if isinstance(sig, ResetSignal):
            sig = self.clock_domains[sig.cd].rst
            if sig is None:
                raise ValueError("Attempted to obtain name of non-existent "
                                 "reset signal of domain "+sig.cd)

        if sig.name_override is not None:
            sig_name = sig.name_override
        else:
            sig_name = self.pnd[sig]
        try:
            n = self.sigs[sig]
        except KeyError:
            try:
                n = self.counts[sig_name]
            except KeyError:
                n = 0
            self.sigs[sig] = n
            self.counts[sig_name] = n + 1
        if n:
            return sig_name + "_" + str(n)
        else:
            return sig_name



from migen.build.generic_platform import Subsignal, Pins, IOStandard
from migen.build.lattice import LatticePlatform
from migen.build.lattice.programmer import MyStormProgrammer


_io = [
    ("sram", 0,
        Subsignal("adr", Pins("137 138 139 141 142 42 43 44 73 74 75 76 115",
                              "116 117 118 119 78")),
        Subsignal("dat", Pins("136 135 134 130 125 124 122 121 62 61 60 56 55",
                              "48 47 45")),
        Subsignal("oe", Pins("29")),
        Subsignal("we", Pins("120")),
        Subsignal("cs", Pins("23")),
        Subsignal("ub", Pins("28")),
        Subsignal("lb", Pins("24")),
        IOStandard("LVCMOS33"),
    ),

    ("clk100", 0, Pins("129"), IOStandard("LVCMOS33")),

    ("mmc", 0,
        Subsignal("dat", Pins("63 64 39 38")),
        Subsignal("cmd", Pins("41")),
        Subsignal("clk", Pins("37")),
        IOStandard("LVCMOS33"),
    ),

    ("serial", 0,
        Subsignal("rx", Pins("88")),
        Subsignal("tx", Pins("85")),
        Subsignal("rts", Pins("91")),
        Subsignal("cts", Pins("94")),
        IOStandard("LVCMOS33"),
    ),

    ("user_btn", 0, Pins("63"), IOStandard("LVCMOS33")),
    ("user_btn", 1, Pins("64"), IOStandard("LVCMOS33")),

    ("user_sw", 0, Pins("37"), IOStandard("LVCMOS33")),
    ("user_sw", 1, Pins("38"), IOStandard("LVCMOS33")),
    ("user_sw", 2, Pins("39"), IOStandard("LVCMOS33")),
    ("user_sw", 3, Pins("41"), IOStandard("LVCMOS33")),

    ("user_led", 0, Pins("71"), IOStandard("LVCMOS33")),
    ("user_led", 1, Pins("67"), IOStandard("LVCMOS33")),
    ("user_led", 2, Pins("68"), IOStandard("LVCMOS33")),
    ("user_led", 3, Pins("70"), IOStandard("LVCMOS33")),

    ("done", 0, Pins("52"), IOStandard("LVCMOS33")),
    ("dbg1", 0, Pins("49"), IOStandard("LVCMOS33")),
    ("greset", 0, Pins("128"), IOStandard("LVCMOS33")),
]

_connectors = [
    ("pmod0", "94 91 88 85"),
    ("pmod1", "95 93 90 87"),
    ("pmod2", "105 102 99 97"),
    ("pmod3", "104 101 98 96"),
    ("pmod4", "143 114 112 107"),
    ("pmod5", "144 113 110 106"),
    ("pmod6", "10 9 2 1"),
    ("pmod7", "8 7 4 3"),
    ("pmod8", "20 19 16 15"),
    ("pmod9", "18 17 12 11"),
    ("pmod10", "34 33 22 21"),
    ("pmod11", "32 31 26 25"),
    ("pmod12", "37 38 39 41"),
    ("pmod13", "71 67 68 70"),
]


class Platform(LatticePlatform):
    default_clk_name = "clk100"
    default_clk_period = 10

    def __init__(self):
        LatticePlatform.__init__(self, "ice40-hx8k-tq144:4k",
                                 _io, _connectors, toolchain="icestorm")

    def create_programmer(self, serial_port="/dev/ttyACM0"):
        return MyStormProgrammer(serial_port)
